
Copyright   S. J. Jones and H. K. D. H. Bhadeshia, 1996
C           University of Cambridge
C           Department of Materials Science and Metallurgy
C           Pembroke Street, Cambridge CB2 3QZ, U. K.
C
C
C Simultaneous transformation to allotriomorphic, idiomorphic, Widmanstatten
C    ferrite and pearlite during slow cooling or isothermal transformation
C Nucleation is allowed either at austenite grain boundaries or on inclusions.
C
C Full details in Acta Metallurgica et Materialia,
C    volume 45 (1997) pp. 2911-2920
C    by S. J. Jones and H. K. D. H. Bhadeshia
C
C Version 2.0, September 1997
C
C Version 2.1, June 2003
C correcting bugs of PEARLOMEGA when used on Linux.
C  by T. Okumura
C
C Version 2.2, August 2003
C  Including Cu, correcting function PEARL, EQGA0
C  by T. Okumura
C
C Version 3.0, September 2003
C  Correct MAP_SIM_TRANS

        IMPLICIT DOUBLE PRECISION (A-H,K-Z), INTEGER (I-J)
        DOUBLE PRECISION VIA(4001),VIINCA(4001),PARA1(4001),PARA3(4001),
     &        DIFFUSY(4001),GMAX(4001),C(9),BC(9),DC(9),
     &        DIF(52),EC(9),NEWEC(9),GMAX1VOL(4001),
     &       VIWID(4001),VIINCWID(4001),QMOL,
     &        STCTEMP(40),STXEQ(40),STXEQ2(40),
     &        GRWID(4001),RIAGBSS(4001),RIVINCSS(4001),
     &        GMAXINC(4001),WIDIAREASS(4001),WIDIVOLSS(4001),
     &        WIDINCIAREASS(4001),WIDINCIVOLSS(4001),
     &       VIINCPEARL(4001),GRPEARL(4001),
     &        VIPEARL(4001),RIVPEARLINCSS(4001),RIAPEARLGBSS(4001),
     &       OREA(4001),OREWID(4001),OREPEARL(4001)
C
C Open file to store the ouput data
C
        OPEN(10,FILE='GBINCSIM.DAT')
C
C Write column titles
C
        WRITE(10,17) 'Time','Trantemp','Percentrans',
     &  'Vareal','Vwidreal','Vpearlreal','Varealinc','Vwidrealinc',
     &  'Vpearlrealinc','Vtotareal','Vtotwidreal','Vtotpearlreal',
     &  'Vtotferrite','Vmartretaus','Cinf','Theory'
17      FORMAT(A,1X,A,1X,A,1X,A,1X,A,1X,A,1X,A,1X,A,1X,A,
     &  1X,A,1X,A,1X,A,1X,A,1X,A,1X,A,1X,A)
C
C
C CONSTANTS: ----------------
C
C       Program version
             AVER=3.0
C
C       the value of pi
             PI=3.141592654
C       Universal gas constant, J/mol/K
             GASC=8.31435
C
C       Boltzmann & Planck constants, Avogadro's number
             BOLT=1.380622D-23
             PLAN=6.626196D-34
             NA=6.022169D23
C
             AMU=1.6605D-27
C
C       density of steel in kg/m**3
            DENSTEEL=7860.0
C
C       austenite/ferrite interface energy J/m**2
            UFAFENERGY=UFAFENERGY
C Aspect ratio of each phase
        ARWID=0.05
        AR=3.0
        ARPEARL=1.0
C
        VOLTOT=1.0
        TOTCARBFERR=0.0
        MODNUCPEARL=0.01
C
C CONSTANTS: ---------------- completed
C
C
C READ IN DATA: -------------
C
        WRITE(*,42)
        CALL RCOMP(C,J6,0)
C
        WRITE(*,*) "Initial time step (s) ?"
           READ(*,*) TIMESTEPINITIAL
C
        WRITE(*,*) "Maximum number of iterations allowed ?"
           READ(*,*) INOSTEPS
C
        WRITE(*,*) "Austenite-ferrite interfacial energy (J/m**2) ?"
           READ(*,*) UFAFENERGY
C
        WRITE(*,*) "Activation energy for transfer of atoms (J/mole) ?"
           READ(*,*) QQ
           QMOL=QQ
           QQ=(QQ*1.0D3)/NA
C
        WRITE(*,*) "Recon. shape factor - boundary nucleation ?"
           READ(*,*) SHAPEGB
C
        WRITE(*,*) "Equiaxed prior austenite grain size (m) ?"
           READ(*,*) MLI
C
        WRITE(*,*) "Fraction of effective boundary sites ?"
           READ(*,*) FRAC
C
        WRITE(*,*) "Recon. shape factor - nucleation on an inclusion ?"
           READ(*,*) SHAPEINC
C
        WRITE(*,*) "Mean inclusion diameter (m) ?"
           READ(*,*) DIAMEAN
C
        WRITE(*,*) "Total volume fraction of inclusions ?"
           READ(*,*) VINCFRAC

        WRITE(*,*) "Fraction of effective inclusions ?"
           READ(*,*) ACTFRACINC
C
        WRITE(*,*) "Analytical comparison - advanced use (0=NO) ?"
           READ(*,*) ITH
C
        WRITE(*,*) "Cooling (1=natural, 2=linear ,3=isothermal) ?"
           READ(*,*) IHEAT
C
C
        IF (IHEAT.EQ.1) THEN
            WRITE(*,*) "Total heat input (kJ/m) ?"
               READ(*,*) HETIN
            WRITE(*,*) "Interpass temperature (centigrade) ?"
               READ(*,*) TEMPINTP
        ELSEIF (IHEAT.EQ.2) THEN
            WRITE(*,*) "Linear cooling rate (centigrade/minute) ?"
               READ(*,*) COOLRATE
        ELSEIF (IHEAT.EQ.3) THEN
            WRITE(*,*) "Isothermal temperature (centigrade) ?"
               READ(*,*) TEMPISOTHERMAL
        ELSE
           WRITE(*,*) "ERROR - incorrect input !"
               GOTO 9999
        ENDIF 
C
C READ IN DATA: -------------complete
C
C
C Calculate phase diagram and thermodynamics: -------------
C
        DO 3520 ICOMP=1,9
         NEWEC(ICOMP)=C(ICOMP)
3520    CONTINUE

        CALL MUCG(NEWEC,WS,BS,MS,T10,T20,W,STCTEMP,
     &  STXEQ,STXEQ2)
C
C Calculate phase diagram and thermodynamics: -------------complete 
C
        WRITE(*,42)
        WRITE(*,46)
        WRITE(*,43) AVER
        WRITE(*,44)
        WRITE(*,45)
        WRITE(*,46)
42      FORMAT(////)
43      FORMAT(//,7X,'Predicted Simultaneous Transformation Kinetics'
     &               ,F4.1)
C CHANGE COPYWRITE SEP/2003
44      FORMAT(/,9X,'S. J. Jones, H. K. D. H. Bhadeshia and T. Okumura')
45      FORMAT(/,17X,'University of Cambridge',//)
46      FORMAT('--------------------------------------------------',
     &  '-----------')
C
C
        WRITE(*,18) 'TIME','TEMPER','ALLIDI','WIDMAN',
     &  'PEARLI','TOTAL','CARBON'
18      FORMAT(9X,A,2X,A,2X,A,2X,A,2X,A,2X,A,3X,A)
        DO 321 I321=1,40
         IF(STCTEMP(I321).EQ.0.0) THEN
          MAXPHTEMPC=STCTEMP(I321-1)
          GOTO 421
         ENDIF
         IF(I321.EQ.40) THEN
          MAXPHTEMPC=STCTEMP(40)
          GOTO 421 
         ENDIF
321     CONTINUE
421     STOPTEMP=STCTEMP(1)   
C  
C Estimate the paraequilibrium Ae3' temperature:
C
          DO 821 ICOMP=1,9
               EC(ICOMP)=C(ICOMP)
821       CONTINUE
C
          CALL OMEGA(EC,W,FRED,T10,T20,0)
          WRITE(10,*) EC(1)
          CALL HIGH(EC,W,T10,T20,PAE3)
          WRITE(10,*) EC(1),PAE3
          IF (PAE3.GE.MAXPHTEMPC) THEN
               STARTTEMP=MAXPHTEMPC
           ELSE
               STARTTEMP=PAE3
           ENDIF      
C
C   
C IHEAT defines the type of heat treatment. 1=natural cooling,
C       2=constant cooling rate, 3=isothermal heat treatment
C
       IF (IHEAT.EQ.1) THEN
        C1=4359.0
        C2=1.510
        HEFFIC=0.95
        HEATIN=HEFFIC*HETIN*29.0/(9.2D-3)
       ELSEIF (IHEAT.EQ.2) THEN 
        COOLRATE=COOLRATE/60.0
       ELSEIF (IHEAT.EQ.3) THEN
        TEMPISOTHERMAL=TEMPISOTHERMAL
       ENDIF      
C
C 
C The austenite grain surface per unit voulme (OBOUND) related to the 
C     grain size as given by the mean lineal intercept (m):
C
       OBOUND=2.0/MLI 
C
C Enrichment of residual austenite with carbon:
C        CINF=CARBON CONCENTRATION OF UNTRANSFORMED AUSTENITE AT ANY INSTANT.
C        CBAR=CARBON CONCENTRATION OF AUSTENITE PRIOR TO TRANSFORMATION.
C        CXYX=CARBON CONCENTRATION OF FERRITE.
C
       CBAR=C(1)/100.0
       IFLAG=0
       IFLAGVTOT=0
C MAIN DO LOOP - USED TO INCREMENT THE CURRENT TIME:
       DO 2 I2=1,INOSTEPS,1
C SET TIME STEP TO BE USED ON FIRST LOOP:
        IF (I2.EQ.1) DTIME=TIMESTEPINITIAL
C IF PEARLITE IS ABOUT TO FORM THEN REDUCE THE TIME STEP:
        IF (VTOTFERRITE.GT.0.95) IFLAGVTOT=IFLAGVTOT+1
        IF ((IFLAGPEARLSOON.EQ.1).OR.(IFLAGVTOT.EQ.1))
     &  DTIME=DTIME/100.0
C INCREASE THE SIZE OF THE TIME STEP:
         DTIME=DTIME*1.04                          
C STORE THE PREVIOUS TIME AND CALCULATE THE CURRENT TIME:
        PREPRETIME=PRETIME
        PRETIME=TIME
        TIME=TIME+DTIME
C STORE TEMPERATURE OF PREVIOUS ISOTHERMAL STEP:
        PREPRETRANTEMP=PRETRANTEMP
        PRETRANTEMP=TRANTEMP
C IHEAT=1: NATURAL WELD COOLING MODEL BASED ON THE WORK OF BHADESHIA WITH NEW CONSTANTS
C FITTED TO BRITISH STEEL EXPERIMENTAL WELDS BY SJ JONES:
        IF (IHEAT.EQ.1) THEN     
         IF (I2.EQ.1) THEN
          HIGHSTEP=STARTTEMP
         ELSE
          HIGHSTEP=LOWSTEP
         ENDIF
         LOWSTEP=FLOWSTEP(HIGHSTEP,DTIME,HEATIN,C1,C2,TEMPINTP)
         TRANTEMP=(HIGHSTEP+LOWSTEP)/2.0
C IHEAT=2: LINEAR COOLING:
        ELSEIF (IHEAT.EQ.2) THEN
         IF (I2.EQ.1) THEN
          HIGHSTEP2=STARTTEMP
         ELSE
          HIGHSTEP2=LOWSTEP2
         ENDIF
         LOWSTEP2=STARTTEMP-(COOLRATE*TIME)
         TRANTEMP=(HIGHSTEP2+LOWSTEP2)/2.0
C IHEAT=3: ISOTHERMAL TREATMENT:
        ELSEIF (IHEAT.EQ.3) THEN
         TRANTEMP=TEMPISOTHERMAL
        ENDIF
C ABSOLUTE TEMPERATURE:
       ABSTEMP=TRANTEMP+273.0    
C
C Calculate nucleation rates, carbon diffusivity and parabolic thickening
C   rate constants appropriate to the current temperature and the far-field
C   carbon concentration of the residual austenite      
C
C Interpolate calculated phase diagram to the current temperature
        CALL INTERPOL(STXEQ,STCTEMP,TRANTEMP,XEQ)
        CALL MFTOWGT(C,XEQ,WXEQ)
           CXYY=WXEQ/100.0
           MOLCXYX=XALPH(ABSTEMP)
        CALL MFTOWGT(C,MOLCXYX,WGTCXYX)
           CXYX=WGTCXYX/100.0
C
C Calculate paraequilibrium fraction of ferrite
C
        PEQVOLFRACFERR=(CXYY-CBAR)/(CXYY-CXYX)
C
C Calculate the weighted average carbon diffusivity at the current ------
C   temperature over the range CXYY to CINF
C   Trivedi and Pound, Journal of Applied Physics, 38 (1967) 3569
C   Bhadeshia, Metal Science, 15 (1981) 477
C   Bhadeshia, Progress in Materials Science,  29 (1985) 321.
C
        IF (I2.EQ.1) THEN
         CINF=CBAR
        ENDIF
        CSTART=CXYY
        CEND=CINF
        ICARBON=50
        CINT=(CSTART-CEND)/(1.0*ICARBON)
        DO 456 I456=1,(ICARBON+1),1
         X=CSTART-(CINT*(I456-1))
         BC(1)=X
         DO 520 ICOMP=2,9
          IF(ICOMP .EQ. 5 .OR. ICOMP .EQ. 6 .OR. ICOMP .EQ. 7 .OR.
     &  ICOMP.EQ.8 ) THEN
           BC(ICOMP)=0.0
          ELSE
           BC(ICOMP)=C(ICOMP)
          ENDIF
520      CONTINUE
         CALL OMEGA(BC,WDIFF,XBARD,T10D,T20D,0)
         IF (I456.EQ.1) THEN
           CSTARTMOL=BC(1)
         ELSEIF (I456.EQ.(ICARBON+1)) THEN
           CENDMOL=BC(1)
         ENDIF
         CALL DIFFUS(DIFC,BC(1),ABSTEMP,WDIFF)
         DIF(I456)=DIFC*0.0001
         IF (DIF(I456).LT.0.0) THEN
            DIF(I456)=DIF(I456-5)
            DIF(I456-1)=DIF(I456-5)
            DIF(I456-2)=DIF(I456-5)
            DIF(I456-3)=DIF(I456-5)
            DIF(I456-4)=DIF(I456-5)
         ENDIF
456     CONTINUE
        CINTMOL=(CSTARTMOL-CENDMOL)/ICARBON
        SUMCAR=0.0       
        DO 543 I543=1,ICARBON,1
         SUMCAR=SUMCAR+(DIF(I543)*CINTMOL)
543     CONTINUE
        DIFFUSY(I2)=(1.0/(CSTARTMOL-CENDMOL))*SUMCAR
C
C Diffusion calculation complete -------------------------------------------
C
C Parameters for three-dimensional parabolic thickening rate constant
C   Bhadeshia, Progress in Materials Science,  29 (1985) 321.
C
 
         IF (CINF.GE.CXYY) THEN
          PARA3(I2)=0.0
         ELSE
          PARA3(I2)=( (2.0*(CINF-CXYY))/(CXYX-CINF) )**0.5
         ENDIF         
         IF (PARA3(I2).LT.0.0) THEN
          PAUSE
         ENDIF
C
C Parameters for one-dimensional parabolic thickening rate constant
C
         IF (CINF.GE.CXYY) THEN
          PARA1(I2)=0.0
         ELSE
          PARA1(I2)=(((CINF-CXYY)**2.0)/((CXYX-CXYY)*(CXYX-CINF)))
     &    **0.5
         ENDIF         
         IF (PARA1(I2).LT.0.0) THEN
          PAUSE
         ENDIF
C
C Reset the composition to weight %
C
         DC(1)=CINF*100.0
          DO 1752 ICOMP=2,9
            DC(ICOMP)=C(ICOMP)
1752      CONTINUE   

C    
C CALCULATE VARIOUS THERMODYNAMIC DRIVING FORCES:
C GMAX=MAXIMUM POSSIBLE DRIVING FORCE FOR PARAEQUILIBRIUM TRANSFORMATION DURING NUCLEATION.
C GMAX=DRIVING FORCE FOR PARAEQUILIBRIUM TRANSFORMATION AFTER ALLOWING FOR 50 J/MOLE OF
C STORED STRAIN ENERGY IN THE FERRITE.
       CALL OMEGA(DC,W,XBAR,T10,T20,0)
       CALL AG(DC,W,T10,T20,ABSTEMP,GMAX(I2),FPRO)
       W1=48570.0
       H=38575.0
       S=13.48
       IF (T.GT.1000.0) THEN
        H1=105525.0D+00
        S1=45.34521D+00
       ELSE
        H1=111918.0D+00
        S1=51.44D+00
       ENDIF
       F=ENERGY(ABSTEMP,T10,T20)
       AJ=1.0-DEXP(-W/(GASC*ABSTEMP))
       AJ1=1.0-DEXP(-W1/(GASC*ABSTEMP))
C CALCULATE THE DRIVING FORCE FTO FOR DIFFUSIONLESS TRANSFORMATION TO FERRITE:
       FTO=FTO1(H,S,DC(1),ABSTEMP,W,W1,H1,S1,F,AJ,AJ1)
       IF (FTO.GT.0) FTO=0.0
C DETERMINE WHETHER OR NOT WIDMANSTATTEN FERRITE OR BAINITE CAN FORM:   
        WIDMSTE=-50.0
        BAINSTE=-400.0
C WIDMANSTTEN FERRITE GROWTH INVOLVES DIFFUSION FPRO
        CALL DISPSTART(WTICKN,WIDMSTE,ABSTEMP,GMAX(I2),FPRO)
C BAINITE GROWTH IS DIFFUSIONLESS IN THE FIRST DISPLACIVE STAGE FTO
        CALL DISPSTART(BTICKN,BAINSTE,ABSTEMP,GMAX(I2),FTO)
C SET A FLAG TO DENOTE THAT WIDM. FERRITE HAS NUCLEATE FOR THE FIRST TIME.
        IF (WTICKN.EQ.1.0) THEN
         WIDGO=1.0
        ENDIF
C BAINITE FORMATION IS NOT CONSIDERED IN THIS MODEL:
        IF (BTICKN.EQ.1.0) THEN
         WRITE(*,*) "WARNING BAINITE FORMATION IS POSSIBLE"
         WRITE(*,*) "THIS IS NOT ALLOWED FOR IN THIS MODEL"
         PAUSE
        ENDIF
        IF (WIDGO.EQ.1.0) THEN
C DETERMINE WHETHER IT IS POSSIBLE FOR WIDMANSTATTEN FERRITE TO GROW IN THIS TIME INTERVAL:
         CALL INTERPOL(STXEQ2,STCTEMP,TRANTEMP,XMAXWID)
         CALL MFTOWGT(C,XMAXWID,WXMAXWID)
         CXMAXWID=WXMAXWID/100.0
         IF ((CINF.GE.CXMAXWID).OR.(FPRO.GE.WIDMSTE)) THEN
C PREVENT FURTHER GROWTH OF WIDMANSTATTEN FERRITE DURING THIS TIME INTERVAL:
          WTICKG=2.0
         ELSE
C WIDMANSTATTEN FERRITE CAN GROW DURING THIS TIME INTERVAL:
          WTICKG=1.0
         ENDIF
        ENDIF 
C MODEL FOR PEARLITE FORMATION: DETERMINE WHETHER OR NOT PEARLITE CAN FORM AND ITS LINEAR
C GROWTH RATE (ASSUME PARAEQUILIBRIUM CONDITIONS - UNREALISTIC):
            W1=CINF*100.0
            W2=C(2)
            W3=C(3)
            W4=C(4)
            W5=C(6)
            W6=C(5)
C INCLUDE COPPER SEP/2003 BY T. OKUMURA
C            W7=0
            W7=C(8)
            IPARA=1
            JPEARL=1
            ISAMPLE=1
            TI=TRANTEMP
        CALL PEARL(W1,W2,W3,W4,W5,W6,W7,TI,ISAMPLE,IPARA,
     &  PX1GC,PX2GC,PX1CG,PX2CG,PX1GA,PX2GA,PX1AG,PX2AG,VEL1,
     &  X1GC,X2GC,X1CG,X2CG, X1GA,X2GA,X1AG,X2AG, 
     &  VEL2,SC,S,JPEARL)
        IF(IPARA .EQ. 1)THEN
         VEL=VEL1*1.0D-2
        ELSE
         VEL=VEL2*1.0D-2
        ENDIF
        CALL MFTOWGT(C,PX1GC,WPX1GC)
        CXMINPEAR=WPX1GC/100.0
        IF (CINF.GE.(0.93*CXMINPEAR)) THEN
         IFLAGPEARLSOON=IFLAGPEARLSOON+1
        ENDIF
        IF (CINF.GE.CXMINPEAR) THEN
         PEARLTICK=1.0
        ELSE
         PEARLTICK=0.0
        ENDIF
C CALCULATE THE MARTENSITE START TEMPERATURE OF THE RESDIUAL AUSTENITE:
        XMS=DC(1)
        IF(XMS .GT. 0.0594D+00)XMS=0.0594D+00
        MARTENSTE=-1086.9D+00 + 2941.8D+00*XMS - 9.6016D+05*XMS*XMS
     &  + 2.5382D+07*XMS*XMS*XMS - 1.8488D+08*XMS*XMS*XMS*XMS
        IF (FTO.LT.MARTENSTE) THEN
         MTICK=1.0
         WRITE(*,*) "BELOW MS TEMPERATURE OF RESIDUAL AUSTENITE"
         GOTO 3769
        ELSE
         MTICK=0.0
        ENDIF
C CALCULATE THE NUCLEATION RATES OF ALLOTRIOMORPHIC AND IDIOMORPHIC FERRITE:
        MASSFEATOM=55.85*1.6605D-27
	GMAX1VOL(I2)=(GMAX(I2)/NA)*(DENSTEEL/MASSFEATOM)
	AEBHOMOG=16.0*PI/3.0*(UFAFENERGY**3)/
     &  (GMAX1VOL(I2)**2.0)
        AEBGB=SHAPEGB*AEBHOMOG
        AEBINC=SHAPEINC*AEBHOMOG
        DIAMEAN=DIAMEAN
        VINCFRAC=VINCFRAC
        NOINC1VOL=VINCFRAC/( (4.0/3.0)*3.141593*((DIAMEAN/2.0)**3.0) )
        RINCSITES1VOL=NOINC1VOL*ACTFRACINC
        TOTINCAREA1VOL=NOINC1VOL*4.0*3.141593*((DIAMEAN/2.0)**2.0)        
        GBSITES1AREA=FRAC*(1.0/((0.287D-9)**2.0))
        GBSITES1VOL=GBSITES1AREA*OBOUND
        SITERATIO=RINCSITES1VOL/GBSITES1VOL
        FF=BOLT*ABSTEMP/PLAN
        CONET=-1.0*BOLT*ABSTEMP
        ETGB=((AEBGB+QQ)/CONET)
        ETINC=((AEBINC+QQ)/CONET)
C STEADY STATE NUCLEATION RATE OF IDIOMORPHIC FERRITE PER UNIT VOL PER UNIT TIME ON
C INCLUSIONS:
        IF (ETINC.GE.-610.0) THEN
         RIVINCSS(I2)=RINCSITES1VOL*FF*DEXP(ETINC)
        ELSE
         RIVINCSS(I2)=0.0
        ENDIF       
C STEADY STATE NUCLEATION RATE OF ALLOTRIOMORPHIC FERRITE PER UNIT AREA PER UNIT TIME ON
C BOUNDARY:
        IF (ETGB.GE.-610.0) THEN
         RIAGBSS(I2)=GBSITES1AREA*FF*DEXP(ETGB)
        ELSE
         RIAGBSS(I2)=0.0
        ENDIF
        RIVGBSS=RIAGBSS(I2)*OBOUND
        IF ((RIVINCSS(I2).EQ.0.0).AND.(RIVGBSS.EQ.0.0)) THEN
         RATIOIVSS=0.0
        ELSE
         RATIOIVSS=RIVINCSS(I2)/RIVGBSS
        ENDIF
C CONVERT STEADY STATE NUCLEATION RATES TO TIME DEPENDENT RATES (CHRISTIAN, 1975):
        RCRIT=-2.0*UFAFENERGY/GMAX1VOL(I2)
        VOLCRIT=4.0*PI*(RCRIT**3.0)/3.0
        VOLFEFER=((0.287D-9)**3.0)/2.0
        NCRITVOL=VOLCRIT/VOLFEFER
        FREQFAC=BOLT*ABSTEMP/PLAN
        ONAREA=(4.0*PI*(RCRIT**2.0)) / ( (0.287D-9)**2.0 )
        DIFRATEATINT=FREQFAC*ONAREA
     &  *DEXP(-1.0*QMOL*1.0D3/GASC/ABSTEMP)
        TIMENUCINC=(NCRITVOL**2.0)/(4.0*DIFRATEATINT)
        ADJUST=DEXP(-1.0*TIMENUCINC/TIME)
        RIAGBSS(I2)=RIAGBSS(I2)*ADJUST
        RIVINCSS(I2)=RIVINCSS(I2)*ADJUST
C CALCULATE WIDMANSTATTEN FERRITE NUCLEATION RATES AT INCLUSIONS AND THE BOUNDARY:
C BOUNDARY MODEL (BHADESHIA AND REES, 1994):
        CALL WSNUCRATE(WIDIAREASS(I2),WIDIVOLSS(I2),
     &  OBOUND,ABSTEMP,GMAX(I2))
C INCLUSION MODEL (SIMILAR TO BOUNDARY MODEL):
        GMAXINC(I2)=GMAX(I2)
        IF (RINCSITES1VOL.EQ.0.0) THEN
         WIDINCIAREASS(I2)=0.0
         WIDINCIVOLSS(I2)=0.0
        ELSE
         CALL WSNUCRATE(WIDINCIAREASS(I2),WIDINCIVOLSS(I2),
     &   TOTINCAREA1VOL,ABSTEMP,GMAXINC(12))
        ENDIF
C NUCLEATION RATE OF ALLOTRIOMORPHIC FERRITE AT BOUNDARIES:
       VIA(I2)=RIAGBSS(I2)
C NUCLEATION RATE OF PEARLITE AT BOUNDARIES:
       IF (PEARLTICK.EQ.1.0) THEN
        RIAPEARLGBSS(I2)=RIAGBSS(I2)*MODNUCPEARL
       ELSE
        RIAPEARLGBSS(I2)=0.0
       ENDIF
       VIPEARL(I2)=RIAPEARLGBSS(I2)
C NUCLEATION RATE OF WIDMANSTATTEN FERRITE AT BOUNDARIES:
        IF (WTICKN.EQ.0.0) THEN
         VIWID(I2)=0.0
        ELSE
         VIWID(I2)=WIDIAREASS(I2)
        ENDIF      
C NUCLEATION RATE OF ALLOTRIOMORPHIC ON INCLUSIONS:
       VIINCA(I2)=RIVINCSS(I2)
C NUCLEATION RATE OF PEARLITE ON INCLUSIONS:
       IF (PEARLTICK.EQ.1.0) THEN
        RIVPEARLINCSS(I2)=RIVINCSS(I2)*MODNUCPEARL
       ELSE
        RIVPEARLINCSS(I2)=0.0
       ENDIF
       VIINCPEARL(I2)=RIVPEARLINCSS(I2)
C NUCLEATION RATE OF WIDMANSTATTEN FERRITE ON INCLUSIONS:
       IF (WTICKN.EQ.0.0) THEN
        VIINCWID(I2)=0.0
       ELSE 
        VIINCWID(I2)=WIDINCIVOLSS(I2)
       ENDIF
C GROWTH RATE OF PEARLITE: 
       IF (PEARLTICK.EQ.1.0) THEN
        GRPEARL(I2)=VEL
       ELSE
        GRPEARL(I2)=0.0
       ENDIF
C GROWTH RATE OF WIDMANSTATTEN FERRITE:
       IF (WTICKG.EQ.0.0) THEN
        GRWID(I2)=0.0
       ELSEIF (WTICKG.EQ.1.0) THEN
C CALCULATE THE MAXIMUM LINEAR GROWTH RATE USING A MODEL DUE TO BHADESHIA AND JONES
C WHICH IS BASED UPON WORK BY TRIVEDI ET AL.        
C        WDIFF=W
        CMOLRES=DC(1)
        CALL PROGC(VMAX,TRANTEMP,XMAXWID,WDIFF,W,CMOLRES)
        IF (VMAX.LT.0.0) THEN
         ABSOLVMAX=ABS(VMAX)      
         IF (ABSOLVMAX.GT.1.0D-6) THEN
          WRITE(*,*) "ERROR - TRIVEDI"
          PAUSE
         ELSE
          VMAX=0.0
         ENDIF
        ENDIF
        GRWID(I2)=VMAX
       ENDIF

C
C REASSIGN NUCLEATION AND GROWTH DATA TO ALLOW COMPARISON BETWEEN THE VARIOUS ANALYTICAL
C AND NUMERICAL MODELS.
C ITH=0 = DO NOTHING - LET ALL PHASES NUCLEATE AND GROW NORMALLY.
C IN ALL OTHER CASES ONLY ONE PHASE IS ALLOWED TO NUCLEATE AT ONLY ONE TYPE OF SITE. THE
C NUCLEATION AND GROWTH CONDITIONS REMAINING INVARINT THROUGHOUT THE TRANSFORMATION. THIS
C ALLOWS DIRECT COMPARISON BETWEEN THE NUMERICAL MODEL AND THE EQUIVALENT ANALYTICAL MODEL:
C ITH=1 = ALLOTRIOMORPHIC FERRITE NUCLEATES AT THE BOUNDARIES.
C ITH=2 = IDIOMORPHIC FERRITE NUCLEATES ON INCLUSIONS.
C ITH=3 = WIDMANSTATTEN FERRITE NUCLEATES AT THE BOUNDARIES.
C ITH=4 = WIDMANSTATTEN FERRITE NUCLEATES ON INCLUSIONS.
C ITH=5 = PEARLITE NUCLEATES AT THE BOUNDARIES.
C ITH=6 = PEARLITE NUCLEATES ON INCLUSIONS.
        IF (ITH.EQ.1) THEN
         CINF=CBAR     
         DIFFUSY(I2)=DIFFUSY(1)
         PARA1(I2)=PARA1(1)
         PARA3(I2)=0.0
         GRWID(I2)=0.0
         VIA(I2)=VIA(1)
         VIWID(I2)=0.0
         VIINCA(I2)=0.0
         VIINCWID(I2)=0.0
         VIPEARL(I2)=0.0
         VIINCPEARL(I2)=0.0
         GRPEARL(I2)=0.0
        ELSEIF (ITH.EQ.2) THEN
         CINF=CBAR    
         PARA1(I2)=0.0
         DIFFUSY(I2)=DIFFUSY(1)
         PARA3(I2)=PARA3(1)
         GRWID(I2)=0.0
         VIA(I2)=0.0
         VIWID(I2)=0.0
         VIINCA(I2)=VIINCA(1)
         VIINCWID(I2)=0.0
         VIPEARL(I2)=0.0
         VIINCPEARL(I2)=0.0
         GRPEARL(I2)=0.0
        ELSEIF (ITH.EQ.3) THEN
         CINF=CBAR       
         DIFFUSY(I2)=0.0
         PARA1(I2)=0.0
         PARA3(I2)=0.0
         GRWID(I2)=GRWID(1)
         VIA(I2)=0.0
         VIWID(I2)=VIWID(1)
         VIINCA(I2)=0.0
         VIINCWID(I2)=0.0
         VIPEARL(I2)=0.0
         VIINCPEARL(I2)=0.0
         GRPEARL(I2)=0.0
        ELSEIF (ITH.EQ.4) THEN
         CINF=CBAR     
         DIFFUSY(I2)=0.0
         PARA1(I2)=0.0
         DIFFUSY(I2)=0.0
         PARA3(I2)=0.0
         GRWID(I2)=GRWID(1)
         VIA(I2)=0.0
         VIWID(I2)=0.0
         VIINCA(I2)=0.0
         VIINCWID(I2)=VIINCWID(1)
         VIPEARL(I2)=0.0
         VIINCPEARL(I2)=0.0
         GRPEARL(I2)=0.0
        ELSEIF (ITH.EQ.5) THEN
         CINF=CBAR       
         DIFFUSY(I2)=0.0
         PARA1(I2)=0.0
         PARA3(I2)=0.0
         GRWID(I2)=0.0
         VIA(I2)=0.0
         VIWID(I2)=0.0
         VIINCA(I2)=0.0
         VIINCWID(I2)=0.0
         VIPEARL(I2)=VIPEARL(1)
         VIINCPEARL(I2)=0.0
         GRPEARL(I2)=GRPEARL(1)
        ELSEIF (ITH.EQ.6) THEN
         CINF=CBAR      
         DIFFUSY(I2)=0.0
         PARA1(I2)=0.0
         PARA3(I2)=0.0
         GRWID(I2)=0.0
         VIA(I2)=0.0
         VIWID(I2)=0.0
         VIINCA(I2)=0.0
         VIINCWID(I2)=0.0
         VIPEARL(I2)=0.0
         VIINCPEARL(I2)=VIINCPEARL(1)
         GRPEARL(I2)=GRPEARL(1)
        ENDIF
C CORRECT ERRORS IN MAP_SIM_TRANS
C ADD OREA ~ WPEARLEXT
C SEP/2003 BY T. OKUMURA
        CALL MAP_SIM_TRANS(I2,TIME,PRETIME,DTIME,PARA1(I2),
     &  PARA3(I2),DIFFUSY(I2),GRWID(I2),GRPEARL(I2),AR,ARWID,ARPEARL,
     &  VIINCA(I2),VIINCWID(I2),VIINCPEARL(I2),VIA(I2),VIWID(I2),
     &  VIPEARL(I2),VOLTOT,OBOUND,PERCENTRANS,VTOTFERRITE,VAREAL,
     &  VWIDREAL,VPEARLREAL,VAREALINC,VWIDREALINC,
     &  VPEARLREALINC,IFLAG,SATTIME,OREA,OREWID,OREPEARL
     &  VAEXT,VWIDEXT,WPEARLEXT)
C TOTAL VOLUME FRACTION OF ALLOTRIOMORPHIC AND IDIOMORPHIC FERRITE:
        VTOTAREAL=VAREAL+VAREALINC
C TOTAL VOLUME FRACTION OF WIDMANSTATTEN FERRITE: 
        VTOTWIDREAL=VWIDREAL+VWIDREALINC
C TOTAL VOLUME FRACTION OF PEARLITE:
        VTOTPEARLREAL=VPEARLREAL+VPEARLREALINC
C CHECK THE MARTENSITE START TEMPERATURE OF THE UNTRANSFORMED AUSTENITE:
3769    IF (MTICK.EQ.1.0) THEN
         VMARTRETAUS=VTOTFERRITE-VTOTTRANS
         IFINISH=1
         GOTO 7424
        ENDIF
C IF PEARLITE FORMATION POSSIBLE:
        IF (VTOTFERRITE.GE.0.998) THEN
         IFINISH=1
         GOTO 7424
        ENDIF
C IF PEARLITE FORMATION NOT POSSIBLE:
        STOPREAC=PEQVOLFRACFERR*0.998
        IF ((VTOTPEARLREAL.EQ.0.0).AND.
     &  (VTOTFERRITE.GE.STOPREAC)) THEN
         IFINISH=1
         GOTO 7424
        ENDIF
C CALCULATE THE CARBON ENRICHMENT OF THE UNTRANSFORMED AUSTENITE DUE TO ALLOTRIOMORPHIC
C AND WIDMANSTATTEN FERRITE FORMATION:
        TOTCARBFERR=CXYX*(VTOTFERRITE-VTOTPEARLREAL)
        CINF=(CBAR-(TOTCARBFERR))/(VOLTOT-(VTOTFERRITE-VTOTPEARLREAL))
C ANALYTICAL MODELS - ONLY USED FOR TESTING:
        IF (ITH.EQ.1) THEN
C ALLOTRIOMORPHIC FERRITE DISCS NUCLEATING AT THE AUSTENITE GRAIN BOUNDARIES (REED AND
C BHADESHIA):
         N=1000
         A=0.0
         B=1.0
         H=(B-A)/N
         SUM=H*0.5*(FUNCREED(A,VIA(1),AR,PARA1(1),DIFFUSY(1),TIME)+
     &   FUNCREED(B,VIA(1),AR,PARA1(1),DIFFUSY(1),TIME))
         DO 55 I55=1,N-1,1
          X=A+(H*I55)
          CHANGE=H*FUNCREED(X,VIA(1),AR,PARA1(1),DIFFUSY(1),TIME)
          SUM=SUM+CHANGE
55       CONTINUE
         VALPHAEXT=2.0*OBOUND*(PARA1(1)*(DIFFUSY(1)**0.5))
     &   *(TIME**0.5)*SUM
         THEORY=VOLTOT*(1.0-DEXP((-1.0*VALPHAEXT)/VOLTOT))
        ELSEIF (ITH.EQ.2) THEN
C ALLOTRIOMORPHIC FERRITE SPHERES NUCLEATING RANDOMLY ON INCLUSIONS (JOHNSON, MEHL AND
C AVRAMI):
         NEWVAL=(-8.0*PI/15.0)*VIINCA(1)*(PARA3(1)**3.0)
     &   *(DIFFUSY(1)**1.5)*(TIME**2.5)
         THEORY=1.0-DEXP(NEWVAL)
        ELSEIF (ITH.EQ.3) THEN
C WIDMANSTATTEN FERRITE PLATES NUCLEATING AT THE AUSTENITE GRAIN BOUNDARIES (S JONES):
         N=1000
         A=0.0
         B=1.0
         H=(B-A)/N
         SUM=H*0.5*(FUNCPLATE(A,ARWID,VIWID(1),GRWID(1),TIME)+
     &   FUNCPLATE(B,ARWID,VIWID(1),GRWID(1),TIME))
         DO 355 I355=1,N-1,1
          X=A+(H*I355)
          CHANGE=H*FUNCPLATE(X,ARWID,VIWID(1),GRWID(1),TIME)
          SUM=SUM+CHANGE
355      CONTINUE
C GROWTH ONLY OCCURS ON ONE SIDE OF BOUNDARY:
         VALPHAEXT=1.0*(OBOUND*GRWID(1)*TIME)*SUM
         THEORY=VOLTOT*(1.0-DEXP((-1.0*VALPHAEXT)/VOLTOT))
        ELSEIF (ITH.EQ.4) THEN
C WIDMANSTATTEN FERRITE PLATES NUCLEATING RANDOMLY ON INCLUSIONS (S JONES):
         NEWVAL=(-1.0/4.0)*VIINCWID(1)*(GRWID(1)**3)*ARWID*(TIME**4)
         THEORY=1.0-DEXP(NEWVAL)
        ELSEIF (ITH.EQ.5) THEN
C PEARLITE DISCS NUCLEATING AT THE AUSTENITE GRAIN BOUNDARIES (S JONES):
         N=1000
         A=0.0
         B=1.0
         H=(B-A)/N
         SUM=H*0.5*
     &   (FUNCLINDISC(A,ARPEARL,VIPEARL(1),GRPEARL(1),TIME)+
     &   FUNCLINDISC(B,ARPEARL,VIPEARL(1),GRPEARL(1),TIME))
         DO 6355 I6355=1,N-1,1
          X=A+(H*I6355)
          CHANGE=H*FUNCLINDISC(X,ARPEARL,VIPEARL(1),GRPEARL(1),TIME)
          SUM=SUM+CHANGE
6355     CONTINUE
         VALPHAEXT=2.0*(OBOUND*GRPEARL(1)*TIME)*SUM
         THEORY=VOLTOT*(1.0-DEXP((-1.0*VALPHAEXT)/VOLTOT))
        ELSEIF (ITH.EQ.6) THEN
C PEARLITE SPHERES NUCLEATING RANDOMLY ON INCLUSIONS (S JONES):
         NEWVAL=(-1.0*PI/3.0)*VIINCPEARL(I2)
     &   *(GRPEARL(I2)**3)*(TIME**4)
         THEORY=1.0-DEXP(NEWVAL)
        ENDIF
C
C
C OUTPUT DATA TO FILE:
7424    IF (VAREAL.LT.0.1E-99) VAREAL=0.1E-99
        IF (VAREALINC.LT.0.1E-99) VAREALINC=0.1E-99
        IF (VWIDREAL.LT.0.1E-99) VWIDREAL=0.1E-99
        IF (VWIDREALINC.LT.0.1E-99) VWIDREALINC=0.1E-99
        IF (VTOTAREAL.LT.0.1E-99) VTOTAREAL=0.1E-99
        IF (VTOTWIDREAL.LT.0.1E-99) VTOTWIDREAL=0.1E-99
        IF (VTOTFERRITE.LT.0.1E-99) VTOTFERRITE=0.1E-99
        IF (VMARTRETAUS.LT.0.1E-99) VMARTRETAUS=0.1E-99
        IF (ADJUST.LT.0.1E-99) ADJUST=0.1E-99
        IF (RIAGBSS(I2).LT.0.1E-99) RIAGBSS(I2)=0.1E-99
        IF (RIVINCSS(I2).LT.0.1E-99) RIVINCSS(I2)=0.1E-99
        IF (RIAGBSS(I2).GT.0.99E99) RIAGBSS(I2)=0.99E99
        IF (RIVINCSS(I2).GT.0.99E99) RIVINCSS(I2)=0.99E99
        IF (PARA1(I2).LT.0.1E-99) PARA1(I2)=0.1E-99
        IF (PARA3(I2).LT.0.1E-99) PARA3(I2)=0.1E-99
        IF (DIFFUSY(I2).LT.0.1E-99) DIFFUSY(I2)=0.1E-99
        IF (VIA(I2).LT.0.1E-99) VIA(I2)=0.1E-99
        IF (VIWID(I2).LT.0.1E-99) VIWID(I2)=0.1E-99
        IF (VIPEARL(I2).LT.0.1E-99) VIPEARL(I2)=0.1E-99
        IF (GRWID(I2).LT.0.1E-99) GRWID(I2)=0.1E-99
        IF (GRPEARL(I2).LT.0.1E-99) GRPEARL(I2)=0.1E-99
        IF (PERCENTRANS.LT.0.1E-99) PERCENTRANS=0.1E-99
C
        CARB=CINF*100.0    
C  
        WRITE(10,7) TIME,TRANTEMP,PERCENTRANS,VAREAL,VWIDREAL,
     &  VPEARLREAL,VAREALINC,VWIDREALINC,VPEARLREALINC,VTOTAREAL,
     &  VTOTWIDREAL,VTOTPEARLREAL,VTOTFERRITE,VMARTRETAUS,CARB,THEORY   
7       FORMAT(E11.5,1X,E11.5,1X,E11.5,1X,E11.5,1X,E11.5,1X,
     &  E11.5,1X,E11.5,1X,E11.5,1X,E11.5,1X,E11.5,1X,E11.5,
     &  1X,E11.5,1X,E11.5,1X,E11.5,1X,E11.5,1X,E11.5)
C
        WRITE(*,37) TIME,TRANTEMP,VTOTAREAL,VTOTWIDREAL,
     &  VTOTPEARLREAL,VTOTFERRITE,CARB,CINF*100,CXMINPEAR*100,
     &  WTICKN,PEARLTICK
C ,IFLAG
C ,IFLAGPEARLSOON  
37      FORMAT(F13.2,1X,F7.2,1X,F7.4,1X,F7.4,1X,
     &  F7.4,1X,F7.4,1X,F7.4,1X,F7.4,1X,F7.4,1X,
     &  F2.0,1X,F4.0)
C ,1X,I2,1X,I2)
C
        IF ((IFINISH.EQ.1).OR.
     &  (TRANTEMP.LE.STOPTEMP)) THEN
         GOTO 9996
        ENDIF
2       CONTINUE
9996    A=A
9997    FORMAT('--------------------------------------------------',
     &  '--------------------------------------------')
        WRITE(*,9998) 
9998    FORMAT('--------------------------------------------------',
     &  '--------------------------------------------',////)
9999    STOP
        END

C-----------------------------------------------------------------------------
C*****************************************************************************
C-----------------------------------------------------------------------------

C FUNCTION USED TO CALCULATE OF ACTIVITY OF CARBON IN GAMMA:
       DOUBLE PRECISION FUNCTION ACG0(T,X1,X2,E11,E12)
       IMPLICIT REAL*8 (A-H,K-Z)
       ACG0=DEXP(DLOG(X1)+E11*X1+E12*X2)
       RETURN
       END

C FUNCTION WHICH GIVES THE NATURAL LOGARITHM OF THE ACTIVITY OF IRON IN AUSTENITE:
       DOUBLE PRECISION FUNCTION AFEG(XEQ,AJ)
       DOUBLE PRECISION XEQ,AJ,DEQ,TEQ
       DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
       TEQ=5*DLOG((1-XEQ)/(1-2*XEQ))
       TEQ=TEQ+DLOG(((1-2*AJ+(4*AJ-1)*XEQ-DEQ)/(2*AJ*(2*XEQ-1)))**6)
       AFEG=TEQ
       RETURN
       END

C SUBROUTINE WHICH CALCULATES THE VARIOUS THERMODYNAMIC DRIVING FORCES FOR PARAEQUILIBRIUM
C TRANSFORMATION OF AUSTENITE TO FERRITE:
       SUBROUTINE AG(C,W,T10,T20,KTEMP,GMAX,FPRO)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z), INTEGER (I,J)
       DOUBLE PRECISION C(8)
       X1=C(1)
       XA=0.001D+00
       R=8.31432D+00
       W1=48570.0D+00
       H=38575.0D+00
       S=13.48D+00
       XEQ=0.1D+00
       T=KTEMP
       CTEMP=T-273.00D+00
       IF (T .LE. 1000.0) GOTO 20
       H1=105525.0D+00
       S1=45.34521D+00
       GOTO 19
20     H1=111918.0D+00
       S1=51.44D+00
19     F=ENERGY(T,T10,T20)
       AJ=1.0D+00-DEXP(-W/(R*T))
       AJ1=1.0D+00-DEXP(-W1/(R*T))
51     TEQ=R*T*AFEG(XEQ,AJ)-F
       IF (DABS(TEQ) .LT. 1.0) GOTO 52
       ETEQ=DAFEG(XEQ,AJ)*R*T
       XEQ=XEQ-TEQ/ETEQ
       GOTO 51
52     AEQ=CG(XEQ,T,W,R)
       AFEQ=AFEG(XEQ,AJ)
       A=CG(X1,T,W,R)
       AFE=AFEG(X1,AJ)
       FPRO=R*T*(X1*(AEQ-A)+(1-X1)*(AFEQ-AFE))
       CALL GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
       GMAX=R*T*(A1-A)
       RETURN
       END

C SUBROUTINE USED TO FIT A STRAIGHT LINE TO DATA USING LINEAR REGRESSION:
C CONST IS THE BEST FIT SLOPE OF THE REGRESSION LINE.
C CORR IS THE CORRELATION COEFFICIENT.
C J8 IS SET TO THE NUMBER OF X,Y PAIRS INPUT INTO THE SUBROUTINE.
C J9 IS THE NUMBER OF X,Y PAIRS WHICH ARE NOT TO BE INCLUDED IN THE ANALYSIS. ONLY THE 
C FIRST J8-J9 PAIRS ARE ANALYSED.
C JP IS SET TO 1 IF THE REGRESSION RESULTS NEED TO BE PRINTED.
C SLOPE IS THE BEST FIT SLOPE OF THE REGRESSION LINE.
C X IS AN ARRAY OF X VALUES.
C Y IS AN ARRAY OF Y VALUES.
       SUBROUTINE ANALY(J8,J9,CONST,SLOPE,CORR,X,Y,JP)
       IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
       DOUBLE PRECISION X(100),Y(100)
       I1=J8
       J8=J8-J9
       AX=0.0D+00
       AY=0.0D+00
       AX2=0.0D+00
       AY2=0.0D+00
       AXY=0.0D+00
       DO 1 I=1,J8
        AX=AX+X(I)
        AY=AY+Y(I)
        AXY=AXY+X(I)*Y(I)
        AX2=AX2+X(I)*X(I)
        AY2=AY2+Y(I)*Y(I)
1      CONTINUE
       CONST=(AY*AX2-AX*AXY)/(J8*AX2-AX*AX)
       SLOPE=((J8*AXY-AX*AY)/(J8*AX2-AX*AX))
       CORR=(J8*AXY-AX*AY)/(DSQRT((J8*AX2-AX*AX)*
     & (J8*AY2-AY*AY)))
       IF(JP .EQ. 1)WRITE(*,2)CONST,SLOPE,CORR,JP
2      FORMAT('   INTERCEPT=',D12.4,'    SLOPE=',D10.3,
     &'    CORRELATION=',F8.4, I10)
       J8=I1
       RETURN
       END

C SUBROUTINE WHICH FINDS THE COMPOSITION WHEN AUSTENITE AND FERRITE OF THE SAME COMPOSITION
C HAVE THE EQUAL GIBBS FREE ENERGIES:
       SUBROUTINE AXTO(H,S,H1,S1,XTO,T,W,W1,F,AJ,AJ1,STORE,J)
       IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
       J=0
1      DFTO=FTO1(H,S,XTO,T,W,W1,H1,S1,F,AJ,AJ1)+STORE
       J=J+1
       IF (DABS(DFTO) .LE. 10.0D+00) GOTO 2
       G9=G91(XTO,T,W,W1,H1,S1,F,H,S,AJ,AJ1)
       IF (J .GE. 9) GOTO 2
       XTO=XTO-DFTO/G9
       IF (XTO .LE. 1.0D-12) GOTO 3
       GOTO 1
3      XTO=0.0000D+00
2      RETURN
       END

C SUBROUTINE TO TEST IF THE INPUT VALUE IS WITHIN THE ALLOWED RANGE:
       SUBROUTINE BOUND(A,B,C)
       DOUBLE PRECISION A,B,C,D
2      WRITE(*,1)B,C
1      FORMAT(12X,' INPUT VALUE OUT OF BOUNDS'/
     & 14X,' THE LIMITS ARE ',D12.4,' TO ', D12.4///
     & 14X,' INPUT NEW VALUE, OR 999 TO CONTINUE WITH'/
     & 14X,' ORIGINAL RISKY INPUT                     ')
       CALL REED(D)
       IF(D .EQ. 999.0)GOTO 3
       A=D
       IF(A .LT. B .OR. A .GT. C)GOTO 2
3      RETURN
       END

C FUNCTION GIVING THE NATURAL LOGARITHM (ACTIVITY OF CARBON IN AUSTENITE):
       DOUBLE PRECISION FUNCTION CG(X,T,W,R)
       DOUBLE PRECISION J,DG,DUMMY,T,R,W,X
       J=1.0-DEXP(-W/(R*T))
        IF(X .LE. 1.0D-10) THEN
         CG=DLOG(1.0D-10)
        ELSE
         DG=DSQRT(1-2*(1+2*J)*X+(1+8*J)*X*X)
         DUMMY=5*DLOG((1-2*X)/X)+6*W/(R*T)+((38575.0)-(13.48)*T)/(R*T)
         CG=DUMMY+DLOG(((DG-1+3*X)/(DG+1-3*X))**6)
        ENDIF
       RETURN
       END

C SUBROUTINE TO CONVERT WEIGHT % TO MOLE FRACTION OR MOLE FRACTION TO WEIGHT %: 
C N=0 CHANGES FROM ATOMIC FRACTION TO WEIGHT PERCENT, ANY OTHER VALUE OF N CONVERTS WEIGHT
C TO ATOMIC FRACTION.
       SUBROUTINE CONV(N,WT,C)
       IMPLICIT REAL*8 (A-H,O-Z)
       DOUBLE PRECISION AN(8), WT(8), C(8), A(8)
       AN(1)=12.0115D+00
       AN(2)=28.09D+00
       AN(3)=54.94D+00
       AN(4)=58.71D+00
       AN(6)=95.94D+00
       AN(5)=52.00D+00
       AN(7)=63.55D+00
       AN(8)=55.84D+00
       IF (N .EQ. 0) GOTO 1
       WT(8)=100.0D+00-WT(1)-WT(2)-WT(3)-WT(4)-WT(5)-WT(6)-WT(7)
       AT=0.0D+00
       DO 2 I=1,8
        A(I)=WT(I)/AN(I)
        AT=AT+A(I)
2      CONTINUE
       DO 3 I=1,8
        C(I)=WT(I)/AN(I)/AT
3      CONTINUE
       GOTO 4
1      C(8)=1.0D+00-C(1)-C(2)-C(3)-C(4)-C(5)-C(6)-C(7)
       AT=0.0D+00
       DO 5 I=1,8
        A(I)=C(I)*AN(I)
        AT=AT+A(I)
5      CONTINUE
       DO 6 I=1,8
        WT(I)=C(I)*AN(I)/AT*100.0D+00
6      CONTINUE
4      RETURN
       END

C FUNCTION WHICH FINDS THE 'ERROR FUNCTION' OF AN ARGUMENT:
       DOUBLE PRECISION FUNCTION DAERF(Y)
       IMPLICIT REAL*8(A-H,O-Z)
       INTEGER ISW,I
       DOUBLE PRECISION P(5),Q(3),P1(8),Q1(7),P2(5),Q2(4)
       DATA P(1)/-.44422647396874/,
     & P(2)/10.731707253648/,
     & P(3)/15.915606197771/,
     & P(4)/374.81624081284/,
     & P(5)/2.5612422994823D-02/
       DATA Q(1)/17.903143558843/,
     & Q(2)/124.82892031581/,
     & Q(3)/332.17224470532/
       DATA P1(1)/7.2117582508831/,
     & P1(2)/43.162227222057/,
     & P1(3)/152.98928504694/,
     & P1(4)/339.32081673434/,
     & P1(5)/451.91895371187/,
     & P1(6)/300.45926102016/,
     & P1(7)/-1.3686485738272D-07/,
     & P1(8)/.56419551747897/
       DATA Q1(1)/77.000152935229/,
     & Q1(2)/277.58544474399/,
     & Q1(3)/638.98026446563/,
     & Q1(4)/931.35409485061/,
     & Q1(5)/790.95092532790/,
     & Q1(6)/300.45926095698/,
     & Q1(7)/12.782727319629/
       DATA P2(1)/-.22695659353969/,
     & P2(2)/-4.9473091062325D-02/,
     & P2(3)/-2.9961070770354D-03/,
     & P2(4)/-2.2319245973418D-02/,
     & P2(5)/-2.7866130860965D-01/
       DATA Q2(1)/1.0516751070679/,
     & Q2(2)/.19130892610783/,
     & Q2(3)/1.0620923052847D-02/,
     & Q2(4)/1.9873320181714/
       DATA XMIN/1.0D-8/,XLARGE/5.6875D0/
       DATA SSQPI/.56418958354776/
       X=Y
       ISW=1
       IF (X.GE.0.0D0) GO TO 5
       ISW=-1
       X=-X
5      IF (X.LT..477D0) GO TO 10
       IF (X.LE.4.0D0) GO TO 25
       IF (X.LT.XLARGE) GO TO 35
       RES=1.D0
       GO TO 50
10     IF (X.LT.XMIN) GO TO 20
       XSQ=X*X
       XNUM=P(5)
       DO 15 I=1,4
        XNUM=XNUM*XSQ+P(I)
15     CONTINUE
       XDEN=((Q(1)+XSQ)*XSQ+Q(2))*XSQ+Q(3)
       RES=X*XNUM/XDEN
       GO TO 50
20     RES=X*P(4)/Q(3)
       GO TO 50
25     XSQ=X*X
       XNUM=P1(7)*X+P1(8)
       XDEN=X+Q1(7)
       DO 30 I=1,6
        XNUM=XNUM*X+P1(I)
        XDEN=XDEN*X+Q1(I)
30     CONTINUE
       RES=XNUM/XDEN
       GOTO 45
35     XSQ=X*X
       XI=1.0D0/XSQ
       XNUM=P2(4)*XI+P2(5)
       XDEN=XI+Q2(4)
       DO 40 I=1,3
        XNUM=XNUM*XI+P2(I)
        XDEN=XDEN*XI+Q2(I)
40     CONTINUE
       RES=(SSQPI+XI*XNUM/XDEN)/X
45     RES=RES*DEXP(-XSQ)
       RES=1.0D0-RES
50     IF (ISW.EQ.-1) RES=-RES
       DAERF=RES
       RETURN
       END

C FUNCTION USED TO CALCULATE COMPLEMENT OF THE 'ERROR FUNCTION' OF THE ARGUMENT:
       DOUBLE PRECISION FUNCTION DAERFC(A)
       DOUBLE PRECISION A,DAERF
       DAERFC=1.0D+00-DAERF(A)
       RETURN
       END

C DAFEG DAFEG DIFFERENTIAL OF LN ACTIVITY OF IRON IN GAMMA
       DOUBLE PRECISION FUNCTION DAFEG(XEQ,AJ)
       DOUBLE PRECISION ETEQ,ETEQ2,DEQ,XEQ,AJ
       DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
       ETEQ=5*((1/(XEQ-1))+2/(1-2*XEQ))
       ETEQ2=6*((4*AJ-1-(0.5/DEQ)*(-2-4*AJ+2*XEQ+16*XEQ*AJ))
     & /(1-2*AJ+(4*AJ-1)*XEQ-DEQ))+6*(4*AJ/(2*AJ*(2*XEQ-1)))
       DAFEG=ETEQ+ETEQ2
       RETURN
       END

C FUNCTION GIVING THE DIFFERENTIAL OF NATURAL LOGARITHM (ACTIVITY OF CARBON IN AUSTENITE):
       DOUBLE PRECISION FUNCTION DCG(X,T,W,R)
       DOUBLE PRECISION J,DG,DDG,X,T,W,R
       J=1.0-DEXP(-W/(R*T)) 
       DG=DSQRT(1.0-2.0*(1.0+2.0*J)*X+(1.0+8.0*J)*X*X)
       DDG=(0.5/DG)*(-2.0-4.0*J+2.0*X+16.0*J*X)
       DCG=-((10.0/(1.0-2.0*X))+(5.0/X))+6.0*((DDG+3.0)
     & /(DG-1.0+3.0*X
     & )-(DDG-3.0)/(DG+1.0-3.0*X))
       RETURN
       END

C FUNCTION GIVING THE CARBON DIFFUSIVITY IN AUSTENITE:
       DOUBLE PRECISION FUNCTION DIFF(T,X1,X2,W)
       IMPLICIT REAL*8(A-H,K-Y), INTEGER(I,J,Z)
       DOUBLE PRECISION D(300),CARB(300)
       HH=6.6262D-34
       KK=1.38062D-23
       Z=12
       A5=1.0D+00
       R=8.31432D+00
       DASH=(KK*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)
       XINCR=(X1-X2)/300.0D+00
       DO 1 I=1,300
        CARB(I)=X2+(I-1)*XINCR
        X=CARB(I)
        THETA=X/(A5-X)
        ACTIV=CG(X,T,W,R)
        ACTIV=DEXP(ACTIV)
        DACTIV=DCG(X,T,W,R)
        DACTIV=DACTIV*ACTIV
        DACTIV=DACTIV*A5/((A5+THETA)**2)
        SIGMA=A5-DEXP((-(W))/(R*T))
        PSI=ACTIV*(A5+Z*((A5+THETA)/(A5-(A5+Z/2)*THETA+(Z/2)*(A5+Z/2)*
     &  (A5-SIGMA)*THETA*THETA)))+(A5+THETA)*DACTIV
        D(I)=DASH*PSI
1      CONTINUE
       II3=0
       CALL DQSES(XINCR,D,300,ANS,IERR)
       DIFF=ANS/(X1-X2)
       RETURN
       END

C SUBROUTINE THAT FINDS THE DIFFUSIVITY OF CARBON IN AUSTENITE (CM^2/S):
C D=DIFFUSIVITY OF CARBON IN AUSTENITE 
C Z=COORDINATION OF INTERSTIAL SITE 
C PSI=COMPOSITION DEPENDENCE OF DIFFUSION COEFFICIENT 
C THETA=NO. C ATOMS/ NO. FE ATOMS 
C ACTIV=ACTIVITY OF CARBON IN AUSTENITE 
C R=GAS CONSTANT 
C X=MOLE FRACTION OF CARBON 
C T=ABSOLUTE TEMPERATURE 
C SIGMA=SITE EXCLUSION PROBABLITY 
C W=CARBON CARBON INTERACTION ENERGY IN AUSTENITE
        SUBROUTINE DIFFUS(DIF,X,T,WDIFF)
        IMPLICIT REAL*8(A-H,O-Y), INTEGER(I-N,Z) 
        W=WDIFF 
        HH=6.6262D-34 
        BOLTZ=1.38062D-23 
        Z=12 
        A5=1.0D+00 
        R=8.31432D+00 
        DASH=(BOLTZ*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)
        THETA=X/(A5-X)
        ACTIV=CG(X,T,W,R)
        ACTIV=DEXP(ACTIV)
        DACTIV=DCG(X,T,W,R)
        DACTIV=DACTIV*ACTIV
        DACTIV=DACTIV*A5/((A5+THETA)**2)
        SIGMA=A5-DEXP((-(W))/(R*T))
        PSI=ACTIV*(A5+Z*((A5+THETA)/(A5-(A5+Z/2)*THETA+(Z/2)*(A5+Z/2)*
     &  (A5-SIGMA)*THETA*THETA)))+(A5+THETA)*DACTIV
        DIF=DASH*PSI
        RETURN
        END

C SUBROUTINE USED TO FIND IF DISPLACIVE TRANSFORMATION IS POSSIBLE:
       SUBROUTINE DISPSTART(FTICKN,F1,ABSTEMP,GMAX,FPRO)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z), INTEGER(I,J)
C GN (J/MOLE) IS THE UNIVERSAL NUCLEATION FUNCTION.
C F1 IS THE STORED ENERGY FOR FERRITE GROWTH IN J/MOL (BAINITE OR WIDMANSTATTEN FERRITE)
C EQUATION FROM ALI AND BHADESHIA MATERIALS SCIENCE AND TECHNOLOGY AUGUST 1991:
       GN=-2540.0+(3.637*(ABSTEMP-273.18))
       IF ((GMAX .LE. GN) .AND. (FPRO .LT. F1)) THEN
        FTICKN=1.0
       ELSE
        FTICKN=0.0
       ENDIF
       RETURN
       END

C SUBROUTINE WHICH COMPUTES THE VALUE OF THE INTEGRAL OVER AN EQUIDISTANTLY TABULATED
C FUNCTION USING SIMPSON'S RULE (5736-XM7 COPYRIGHT IBM CORPORATION 1971):                           
       SUBROUTINE DQSES(STEP,Y,IDIM,S,IER)
  
       DIMENSION Y(1)
       INTEGER IER,ISW,JMP
       DOUBLE PRECISION STEP,Y,S,A,B,YA,YB
       ISW=IER
       IER=0
       JMP=1
       IF (IDIM) 1,1,2
1      IER=1000
       GOTO 4
2      S=0.D0
       IF (STEP) 6,3,6
3      IER=10
4      IF (ISW+12345) 5,15,5
5      CALL WIER(IER,20523)
       GOTO 15
6      LAST=1+(IDIM-1)*JMP
       A=0.D0
       B=0.D0
       YA=Y(1)
       YB=Y(JMP+1)
       IF (IDIM-2) 15,13,7
7      J=1
       NA=IDIM/2
       IF (NA+NA-IDIM) 9,8,8
8      J=J+JMP
       S=S+1.125D0*YA+2.375D0*YB-0.625D0*Y(2*JMP+1)
     & +0.125D0*Y(3*JMP+1)
9      I=J
10     IF (I-LAST+JMP+JMP) 11,11,12
11     A=A+Y(I)
       IJMP=I+JMP
       B=B+Y(IJMP)
       I=IJMP+JMP
       GOTO 10
12     S=S+A+A+4.D0*B-Y(J)+Y(LAST)
       GOTO 14
13     S=(YA+YB)*1.5D0
14     S=S*STEP/3.D0
15     RETURN
       END

C SUBROUTINE USED TO FIND GIBBS FREE ENERGY FOR THE REACTION FERRITE TO AUSTENITE IN
C SUBSTITUTIONALLY ALLOYED IRON (ZERO CARBON):
       DOUBLE PRECISION FUNCTION ENERGY(T,T10,T20)
       DOUBLE PRECISION T,T10,T20,F,T7,FC
       T7=T-100*T20
       CALL GALGA(F,FC,T7)
       ENERGY=(141.0*T10*4.187)-F
       RETURN
       END

C SUBROUTINE TO CALCULATE THE EQUILIBRIUM CHEMISTRY AT THE AUSTENITE / AUSTENITE+FERRITE
C INTERFACE:
       SUBROUTINE EQGA0(T,X,Y,ACTIV,IM)
       IMPLICIT REAL*8 (A-H,K-Z)
       INTEGER ITER
       DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),K(8),
     & XX(3),G(8),G2(8),G3(8),G4(8),GM(8),G0(8),
     & LFX(4,8),AC(8),E(8,8),EA(8,8),FUN(3)
       COMMON/TRANS/C0,W0,K
       COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC,
     & GFE3C,G3,G4,WW,E,EA,G,G0
       R=8.314D+00
       XGA=C0(1)*1.1D+00
       DX1=1.0D-06
CC CHANGE THE WAY OF CALCULATION AUG/2003
CC NEWTON RAPHSON METHOD CANNOT BE APPLIED IN SOME CASES
CC BECAUSE THERE ARE MORE TAHN ONE MINIMA
C200    XX(1)=XGA-DX1
C       XX(2)=XGA+DX1
C       XX(3)=XGA
       ITER=0
       XX(1)=0.0001
       XX(3)=0.9
200    XX(2)=(XX(1)+XX(3))/2   
       ITER=ITER+1
       DO 220 I=1,3
        X1=XX(I)
        A1=DEXP(G0(1)/R/T+E(1,1)*X1)/
     &  (1.0D+00+EA(1,1)*X1*DEXP(G0(1)/R/T))
        A2=DEXP(G0(IM)/R/T+E(1,IM)*X1)/
     &  (1.0D+00+E(1,IM)*X1*DEXP(G0(1)/R/T))
        Y1=A1*X1
        X2=X2AC0(T,X1,ACTIV,E(1,1),E(1,IM))
        Y2=A2*X2
        X0=1.0D+00-X1-X2
        Y0=1.0D+00-Y1-Y2
        FFG=DLOG(X0)-1.0/2.0*E(1,1)*X1*X1-1.0/2.0*E(IM,IM)*X2*X2
     &  -E(1,IM)*X1*X2
        FFA=DLOG(Y0)-1.0/2.0*EA(1,1)*Y1*Y1-1.0/2.0*EA(IM,IM)*Y2*Y2
     &  -E(1,IM)*Y1*Y2    
        FUN(I)=G4(8)/R/T+FFG-FFA
220    CONTINUE    
       IF (ABS(FUN(3)) .LT. 1.0D-06) GOTO 100
CC SEE ABOVE
C       DUMMY=XGA-2.0*DX1*FUN(3)/(FUN(2)-FUN(1))
C       IF (DUMMY .LT. 0.0D+00) THEN
C        DUMMY=XGA/2.0D+00
C       ENDIF
C       IF (DUMMY .GT. 1.0D+00) THEN
C        DUMMY=(1.0D+00+XGA)/2.0
C       ENDIF
C       XGA=DUMMY
       IF (ITER.GT.10000) GOTO 90
       IF (FUN(2).LT.0.0) THEN
        XX(3)=XX(2)
       ELSE
        XX(1)=XX(2)
       ENDIF
       GOTO 200
 90    WRITE(*,*) 'ITERATION NUMBER BECOMES 10000'
       STOP
100    X(1)=X1
       X(IM)=X2
       Y(1)=Y1
       Y(IM)=Y2
       RETURN
       END

C SUBROUTINE TO CALCULATE THE EQUILIBRIUM CHEMISTRY AT THE AUSTENITE / AUSTENITE+CEMENTITE
C INTERFACE:
       SUBROUTINE EQGC0(T,X,Y,ACTIV,IM)
       IMPLICIT REAL*8 (A-H,K-Z)
       DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),
     & K(8),XX(3),FUN(3),G0(8),
     & G(8),G2(8),G3(8),G4(8),GM(8),MX(8),MM3C(8),
     & LFX(4,8),AC(8),E(8,8),EA(8,8)
       COMMON/TRANS/C0,W0,K
       COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC,
     & GFE3C,G3,G4,WW,E,EA,G,G0
       R=8.314D+00

C MODIFY THE WAY OF CALCULATION
C SEE EQGA0
C       XGC=C0(IM)*0.99D+00
C7      X2=X2AC0(T,XGC,ACTIV,E(1,1),E(1,IM))    
C       IF (X2 .LT. 0.0D+00) THEN
C        XGC=(C0(1)+XGC)/2.0
C        GOTO 7
C       ENDIF    
       ITER=0
       DUMMY=1.0D-6

C MODIFY THE WAY OF CALCULATION
C SEE EQGA0
C100    XX(1)=XGC-DUMMY
C       XX(2)=XGC+DUMMY
C       XX(3)=XGC
        XX(1)=1.0D-6
        XX(2)=0.9
 100    XX(3)=(XX(1)+XX(2))/2
       DO 10 I=1,3
        X1=XX(I)
        FUN(I)=0.0D+00
        B=DEXP((GFE3C-G(IM)-WW(IM))/R/T+E(1,IM)*X1)
        X2=X2AC0(T,X1,ACTIV,E(1,1),E(1,IM))
        Y2=B*X2
        X8=1.0D+00-X1-X2
        Y8=1.0D+00-Y2
        IF (X(8) .LT. 0.0D+00) THEN
         WRITE(6,*) 'X8,X1=',X8,X1
        ENDIF
        MFE3C=R*T*DLOG(Y8)
        MFE=R*T*DLOG(X8)-R*T/2.0*E(1,1)*X1*X1
        MC=R*T/3.0*DLOG(X1)+R*T/3.0*E(1,1)*X1
        MFE3C=MFE3C+(1-Y8)*WW(IM)*Y2
        MFE=MFE-R*T/2.0*E(IM,IM)*X2*X2-R*T*X1*E(1,IM)*X2
        MC=MC+R*T/3.0*E(1,IM)*X2
        MX(IM)= R*T*DLOG(X2)
     &  +R*T*(E(1,IM)*X1+E(IM,IM)*X2)
        MM3C(IM)= R*T*DLOG(Y2)
     &  +Y8*WW(IM)
        MM3C(IM)=MM3C(IM)-Y8*WW(IM)*Y2
        FUN(I)=GFE3C+MFE3C-MFE-MC
10     CONTINUE
       IF(ABS(FUN(3)) .LT. 1.0D-02) GOTO 20
C MODIFY CALCULATION
C       DUMMY1=XGC-2.0*DUMMY*FUN(3)/(FUN(2)-FUN(1))
C       IF (DUMMY1 .LT. 0.0D+00) THEN
C        DUMMY1=XGC/2.0D+00
C       ENDIF
C       XGC=DUMMY1
        IF(FUN(3).LT.0) THEN
         XX(2)=XX(3)
        ELSE
         XX(1)=XX(3)
        ENDIF
         ITER=ITER+1
         IF (ITER.GT.10000) THEN
          WRITE(*,*) 'NO ANSWERE IN EQGC0'
          STOP
         ENDIF
       GOTO 100
20     X(1)=X1
       X(IM)=X2
       Y(1)=0.25D+00
       Y(IM)=3.0D+00/4.0D+00*Y2
       RETURN
       END

C FUNCTION THAT CALCULATES THE TEMPERATURE OF A WELD AFTER COOLING FOR A SPECIFIED
C TIME FROM A SPECIFIED TEMPERATURE:
       FUNCTION FLOWSTEP(HIGHSTEP,DTIME,HEATIN,C1,C2,TEMPINTP)
       DOUBLE PRECISION HIGHSTEP,FLOWSTEP,HEATIN,C1,C2,
     & TEMPINTP,DTIME,TERMA,TERMB,TERMC
       TERMA=(HIGHSTEP-TEMPINTP)**(1.0-C2)
       TERMB=(DTIME)*((C1*(1.0-C2))/HEATIN)
       TERMC=(TERMA-TERMB)**(1.0/(1.0-C2))
       FLOWSTEP=TERMC+TEMPINTP
       RETURN
       END

C FUNCTION USED IN THE ANALYTICAL MODEL OF THE BOUNDARY NUCLEATED PEARLITE TRANSFORMATION:  
       FUNCTION FUNCLINDISC(THETA,ARPEARL,VIA,GROWA,TIME)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z), INTEGER (I-J)
       PI=3.141592654
       TERMA=(-1.0*PI*VIA*((ARPEARL*GROWA)**2.0)*(TIME**3.0)
     & )/3.0
       TERMB=1.0-(THETA**3)
       FUNCLINDISC=1.0-(DEXP(TERMA*TERMB))
       RETURN
       END

C FUNCTION USED IN THE ANALYTICAL MODEL OF THE BOUNDARY NUCLEATED WIDMANSTATTEN FERRITE
C TRANSFORMATION:  
       FUNCTION FUNCPLATE(THETA,BETA,VIA,GROWA,TIME)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z), INTEGER (I-J)
       TERMA=( -1.0*VIA*BETA*(GROWA**2)*(TIME**3) )/3.0
       TERMB=1.0-(THETA**3)
       FUNCPLATE=1.0-(DEXP(TERMA*TERMB))
       RETURN
       END

C FUNCTION USED IN THE ANALYTICAL MODEL OF THE BOUNDARY NUCLEATED ALLOTRIOMOPHIC FERRITE
C TRANSFORMATION: 
       FUNCTION FUNCREED(THETA,VIA,AR,PARAA,DIFFUSY,TIME)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z), INTEGER (I-J)
       ALPHARR=PARAA*(DIFFUSY**0.5)
       TERMA=-0.5*3.1415927*VIA*((AR*ALPHARR)**2)*(TIME**2)
       TERMB=1.0-(THETA**4)
       FUNCREED=1.0-DEXP(TERMA*TERMB)
       RETURN
       END

C FUNCTION USED TO FIND THE GROWTH RATE OF WIDMANSTATTEN FERRITE:
       FUNCTION FUNCTRIV(PECLET,OMEG,RATIO)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z), INTEGER(I,J)
       LHS=OMEG
       S2=-1.073019925D+00*(DLOG10(PECLET))-0.273767575D+00
       S2=10.0D+00**S2
       PI=3.1415927D+00
       IF (PECLET.GT.675.0) THEN
        PECLET=675.0
       ENDIF
       RHS=(DSQRT(PI*PECLET))*(DEXP(PECLET))*(DAERFC(DSQRT(PECLET)))
     & *(1.0D+00+((1.0/RATIO)*OMEG*S2))
       FUNCTRIV=LHS-RHS
       RETURN   
       END

C FUNCTION WHICH FINDS THE THERMODYNAMIC DRIVING FORCE (FTO1) FOR DIFFUSIONLESS
C TRANSFORMATION FROM AUSTENITE TO FERRITE:
       DOUBLE PRECISION FUNCTION FTO1(H,S,X,T,W,W1,H1,S1,F,AJ,AJ1)
       DOUBLE PRECISION R,X,T,T60,ZEN1,ZEN2,ZENER,W,W1,H1,S1,F,
     & AJ,AJ1,D,D11,H,S
       D=DSQRT(1-2*(1+2*AJ)*X+(1+8*AJ)*X*X)
       D11=DSQRT(9-6*X*(2*AJ1+3)+(9+16*AJ1)*X*X)
       T60=T*(1-X)/(28080*X)
       IF (T60 .GT. 1.0) GOTO 18
       IF (T60 .GT. 0.25) GOTO 17
       ZEN1=0.2307+42.7974*T60-233.8631*T60*T60+645.4485*T60*T60*T60
     & -954.3995*(T60**4)+711.8095*(T60**5)-211.5136*(T60**6)
       ZEN2=-2.6702+45.6337*T60-225.3965*T60*T60+567.7112*(T60**3)
     & -771.6466*(T60**4)+538.1778*(T60**5)-151.3818*(T60**6)
       GOTO 16
17     ZEN2=1
       ZEN1=3.295
16     ZENER=(((ZEN2*X)**2)*(-50898.56)/(1-X))+ZEN1*T*X*0.6623741
       ZENER=ZENER*4.187
       GOTO 15
18     ZENER=0.0
15     R=8.31432
       FTO1=X*R*T*DLOG(X*X)+X*(H1-(H)-(S1-(S)
     & )*T+4*W1-6*W)-R*T*(1-X)*DLOG((1-X)**4)+5*R*T*(1-2*X)*DLOG(1-2*X)
     & -R*T*X*DLOG(((D-1+3*X)/(D+1-3*X))**6)-R*T*(1-X)*DLOG(((1-2*AJ+(4
     & *AJ-1)*X-D)/(2*AJ*(2*X-1)))**6)+3*R*T*X*DLOG(3-4*X)+R*T*
     & X*DLOG(((D11-3+5*X)/(D11+3-5*X))**4)+(1-X)*F+ZENER
       RETURN
       END

C FUNCTION WHICH FINDS THE DERIVATIVE OF THE THERMODYNAMIC DRIVING FORCE (FTO1) FOR
C DIFFUSIONLESS TRANSFORMATION FROM AUSTENITE TO FERRITE:
       DOUBLE PRECISION FUNCTION G91(XTO,T,W,W1,H1,S1,F,
     & H,S,AJ,AJ1)
       DOUBLE PRECISION R,XTO,T,W,W1,H1,S1,F,AJ,AJ1,FD,FD1,DT5,
     & DT6,DZEN1,DZEN2,DZEN3,V1,V2,V3,V4,V5,V6,V7,V8,V9,G1,
     & DZEN6,DZEN7,DZEN8,H,S
       R=8.31432
       FD=DSQRT(1-2*(1+2*AJ)*XTO+(1+8*AJ)*XTO*XTO)
       FD1=DSQRT(9-6*XTO*(2*AJ1+3)+(9+16*AJ1)*XTO*XTO)
       DT5=28080*XTO/(1-XTO)
       DT6=T/DT5
       IF (DT6 .GT. 1.0) GOTO 48
       IF (DT6 .LT. 0.25) GOTO 47
       DZEN1=0.2307+42.7974*DT6-233.8631*(DT6**2)+645.4485*(DT6**3)
     & -954.3995*(DT6**4)+711.8095*(DT6**5)-211.5136*(DT6**6)
       DZEN2=-2.6702+45.6337*DT6-225.3965*(DT6**2)+567.7112*(DT6**3)
     & -771.6466*(DT6**4)+538.1778*(DT6**5)-151.3818*(DT6**6)
       GOTO 46
47     DZEN2=1
       DZEN1=3.295
46     DZEN3=(((DZEN2*XTO)**2)*(-50898.56)/(1-XTO))+DZEN1*T*XTO*
     & (0.6623741)
       DZEN3=DZEN3*4.187
       GOTO 45
48     DZEN3=0.0
45     V1=FD-1+3*XTO
       V2=FD+1-3*XTO
       V3=1-2*AJ+(4*AJ-1)*XTO-FD
       V4=2*AJ*(2*XTO-1)
       V5=FD1-3+5*XTO
       V6=FD1+3-5*XTO
       V7=(XTO-1-2*AJ+8*XTO*AJ)/FD
       V8=(9*XTO-9-6*AJ1+16*AJ1*XTO)/FD1
       V9=H1-(H)-(S1-(S))*T-6*W+4*W1
       G1=2+DLOG(XTO**2)+4+DLOG((1-XTO)**4)-10-DLOG((1-2*XTO)**10)
     & -DLOG((V1/V2)**6)-6*((XTO/V1)*(V7+3+V1*(3-V7)/V2))
     & +DLOG((V3/V4)**6)-6*((1-XTO)/V3)*(4*AJ*(1-(V3/V4))-V7)
     & +3*(DLOG(3-4*XTO)-4*XTO/(3-4*XTO))+DLOG((V5/V6)**4)+4*(XTO
     & /V5)*(V8+5+(V5/V6)*(5-V8))
       IF (DT6 .GT. 1.0) GOTO 90
       DZEN6=(-3.3948+13.6112*DT6-13.4376*(DT6**2))*T/(28080*
     & ((1-XTO)**2))
       DZEN7=(-3.3118+15.7462*DT6-23.2449*(DT6**2))*T/(28080*
     & ((1-XTO)**2))
       DZEN8=50898*((DZEN2*XTO)**2)/((1-XTO)**2)+(-50898*(2*
     & DZEN2*DZEN6*(XTO**2)+2*XTO*(DZEN2**2))/(1-XTO))+DZEN1
     & *T*0.6623741+DZEN7*T*XTO*0.6623741
       GOTO 91
90     DZEN8=0.0
91     G91=V9+R*T*G1+DZEN8-F
       RETURN
       END

C SUBROUTINE USED TO FIND THE DIFFERENCE IN GIBBS FREE ENERGY FOR THE REACTION FERRITE TO
C AUSTENITE IN PURE IRON:
       SUBROUTINE GALGA (G,GC,T)
       DOUBLE PRECISION G,GC,T
C METHOD DUE TO REED, DATA DUE TO KAUFMAN, CLOUGHERTY AND WEISS, ACTA MET. 11 1963 P323.
C (WITH CORRECTIONS):
       IF (T.LT.3.56D+02) THEN
        G =  0.13132885D+04 + 1.3141025D-02*T - 2.580128186D-03*T*T
       ENDIF
       IF (T.GT.3.56D+02 .AND. T.LT.4.28D+02) THEN
        G =  1.1224052D+03 + 3.542835682*T - 2.2496774D-02*T*T
     &  + 4.326701963D-05*T*T*T - 3.072096339D-08*T*T*T*T
       ENDIF
       IF (T.GE.4.28D+02 .AND. T.LT.6.34D+02) THEN
        G = - 1.7031057D+04 + 0.1448353906D+03*T - 0.428166357*T*T
     &  + 5.50871582D-04*T*T*T - 2.633397102D-07*T*T*T*T
       ENDIF
       IF (T.GE.6.34D+02 .AND. T.LT.7.40D+02) THEN
        G = 0.13022482D+04 + 0.522110231*T - 5.980360202D-03*T*T
     &  + 6.659625695D-06*T*T*T - 2.473169175D-09*T*T*T*T
       ENDIF
       IF (T.GE.7.40D+02 .AND. T.LT.8.60D+02) THEN
        G = - 0.64489093D+05 + 3.095104796D+02*T - 0.547164618*T*T
     &  + 4.255071973D-04*T*T*T - 1.233061536D-07*T*T*T*T
       ENDIF
       IF (T.GE.8.60D+02 .AND. T.LT.9.27D+02) THEN
        G = + 0.13015055D+07 - 5.763825D+03*T + 9.571875*T*T
     &  - 7.0625D-03*T*T*T + 1.953125D-06*T*T*T*T
       ENDIF
       IF (T.GE.9.27D+02 .AND. T.LT.1.08D+03) THEN
        G = + 0.58257237D+05 - 2.229275965D+02*T + 0.322821329*T*T
     &  - 2.089186484D-04*T*T*T + 5.084325376D-08*T*T*T*T
       ENDIF
       IF (T.GE.1.08D+03 .AND. T.LT.1.24D+03) THEN
        G = 0.14043448D+06 - 4.704834516D+02*T + 0.591517384*T*T
     &  - 3.306547594D-04*T*T*T + 6.93139101D-08*T*T*T*T
       ENDIF
       IF (T.GE.1.24D+03 .AND. T.LT.1.502D+03) THEN
        G = 0.15744376D+05 - 4.475440789D+01*T + 4.7873355D-02*T*T
     6  - 2.286184211D-05*T*T*T + 4.111842105D-09*T*T*T*T
       ENDIF
       IF (T.GE.1.502D+03 .AND. T.LT.1.616D+03) THEN
        G = - 0.58637400D+04 + 1.147433313D+01*T
     &  - 7.5500D-03*T*T + 1.6666666D-06*T*T*T
       ENDIF
       IF (T.GE.1.616D+03 .AND. T.LT.1.68D+03) THEN
        G = 0.32972410D+06 - 8.211649405D+02*T + 0.766426516*T*T
     &  - 3.177724674D-04*T*T*T + 4.938936812D-08*T*T*T*T
       ENDIF
       IF (T.GE.1.68D+03) THEN
        G = 0.33271186D+07 - 7.657409945D+03*T
     &  + 6.607976863*T*T - 2.534098538D-03*T*T*T
     &  + 3.643939352D-07*T*T*T*T
       ENDIF
C CONVERT UNITS TO J/MOLE:
       G = G * 8.314D+00 / 1.9858D+00
       GC = - 6.41114D4 + 3.2158D1*T
       RETURN
       END

C SUBROUTINE USED IN THE CALCULATION OF THE MAXIMUM POSSIBLE DRIVING FORCE (GMAX) FOR THE
C AUSTENITE TO FERRITE TRANSFORMATION:
       SUBROUTINE GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
       IMPLICIT REAL*8(A-H,K-Z)
C CALCULATION OF THE OPTIMUM NUCLEUS CARBON CONTENT AND THE ACTIVITY OF CARBON IN THE
C FERRITE NUCLEUS:
       X=6.3998D-07*T-3.027D-04
       IF(X .LE. 0.0D+00)  X=1.0D-13
       IF(X .GT. 0.32D-03) X=0.32D-03
       AJ1=1.0D+00-DEXP(-W1/(R*T))
1      D1=DSQRT(9.0D+00-6.0D+00*X*(2.0D+00*AJ1+3.0D+00)
     & +(9.0D+00+16.0D+00*AJ1)*X*X)
       B1=((D1-3.0D+00+5.0D+00*X)/(D1+3.0D+00-5.0D+00*X))
       B2=((3.0D+00-4.0D+00*X)/X)
       B3=(H1 - S1*T + 4.0D+00*W1)/(R*T)
       A1=DLOG(B1*B1*B1*B1*B2*B2*B2)+B3
       IF(X .GT. 1.0D-08)THEN
        A1FE=DLOG(1.0D+00-X)
       ELSE
        A1FE=X
       ENDIF
       TEST=F+R*T*(A1FE-AFE) - R*T*(A1-A)
C NEWTON TYPE ITERATION:
       IF (DABS(TEST) .GT. 10.0D-3) THEN
        DA1=(3.0D+00*X/(3.0D+00-4.0D+00*X))*(
     &  (4.0D+00*X-3.0D+00)/(X*X)-4.0D+00/X)
        DA2=(0.5D+00/D1)*(-12.0D+00*AJ1-18.0D+00+
     &  18.0D+00*X+32.0D+00*AJ1*X)
        DA2=4.0D+00*(((DA2+5.0D+00)/(D1-3.0D+00+5.0D+00*X))
     &  -((DA2-5.0D+00)/(D1+3.0D+00-5.0D+00*X)))
        DA1=DA1+DA2
        DA1FE=1.0D+00/(X-1.0D+00)
        ERROR=TEST/(R*T*(DA1FE-DA1))
        IF (ERROR .GT. X) ERROR = 0.3D+00*X
        X=DABS(X-ERROR)
        GOTO 1
       ENDIF
       RETURN
       END

C SUBROUTINE WHICH FINDS THE APPROXIMATE TEMPERATURE AT WHICH PARAEQUILIBRIUM FERRITE
C FORMATION FIRST BECOMES POSSIBLE:
       SUBROUTINE HIGH(C,W,T10,T20,HIGHT)
       IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
       DOUBLE PRECISION C(9)
       X1=C(1)
       XA=0.001D+00
       R=8.31432D+00
       W1=48570.0D+00
       H=38575.0D+00
       S=13.48D+00
       XEQ=0.05D+00
       DO 1 I=1183,573,-1
        KTEMP=I
        T=KTEMP
        IF (T .LE. 1000.0) GOTO 20
        H1=105525.0D+00
        S1=45.34521D+00
        GOTO 19
20      H1=111918.0D+00
        S1=51.44D+00
19      F=ENERGY(T,T10,T20)
        AJ=1.0D+00-DEXP(-W/(R*T))
        AJ1=1.0D+00-DEXP(-W1/(R*T))
51      TEQ=R*T*AFEG(XEQ,AJ)-F
        IF (DABS(TEQ) .LT. 1.0) GOTO 52
        ETEQ=DAFEG(XEQ,AJ)*R*T
        XEQ=XEQ-TEQ/ETEQ
        GOTO 51
52      IF(XEQ .GE. X1) THEN
         HIGHT=KTEMP-273.15D+00
         GOTO 222
        ENDIF
1      CONTINUE
222    RETURN
       END

C SUBROUTINE USED TO FIND A POINT ON A CURVE USING INTERPOLATION:
       SUBROUTINE INTERPOL(YARR,XARR,XVAL,YVAL)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z),INTEGER(I,J)
       DOUBLE PRECISION YARR(51),XARR(51)
       DO 10 I10=1,50,1
        IF ((XARR(I10).LE.XVAL).AND.(XARR(I10+1).GE.XVAL)) THEN
        M=(YARR(I10)-YARR(I10+1))/(XARR(I10)-XARR(I10+1))
        C=YARR(I10)-(M*XARR(I10))
        YVAL=(M*XVAL)+C
        GOTO 20
        ENDIF
10     CONTINUE
20     RETURN
       END
C
C-------------------------------------------------------------------------------
C
C SUBROUTINE TO MODEL THE SIMULTANEOUS TRANSFORMATION KINETICS OF ALLOTRIOMORPHIC,
C IDIOMORPHIC AND WIDMANSTATTEN FERRITE AND PEARLITE. NUCLEATION IS ALLOWED TO
C OCCUR RANDOMLY ON INCLUSIONS AND ON THE AUSTENITE GRAIN BOUNDARIES.

Copyright   S. J. Jones and H. K. D. H. Bhadeshia, 1996
C           University of Cambridge
C           Department of Materials Science and Metallurgy
C           Pembroke Street, Cambridge CB2 3QZ, U. K.
C CORRECTED SEP/2003 BY T. OKUMURA
C
       SUBROUTINE MAP_SIM_TRANS(I2,TIME,PRETIME,DTIME,PARA1,
     &PARA3,DIFFUSY,GRWID,GRPEARL,AR,ARWID,ARPEARL,
     &VIINCA,VIINCWID,VIINCPEARL,VIA,VIWID,VIPEARL,
     &EQVOLFRAC,OBOUND,PERCENTRANS,VTOTTRANS,VAREAL,
     &VWIDREAL,VPEARLREAL,VAREALINC,VWIDREALINC,
     &VPEARLREALINC,IFLAG,SATTIME,OREA,OREWID,OREPEARL
     &VAEXT,VWIDEXT,VPEARLEXT)
C
       IMPLICIT DOUBLE PRECISION (A-H,K-Z), INTEGER (I-J)
       DOUBLE PRECISION
     &QA(1001),PREQA(1001),OREA(3000),PREOREA(3000),QWID(1001),
     &PREQWID(1001),OREWID(3000),PREOREWID(3000),QPEARL(1001),
     &PREQPEARL(1001),OREPEARL(3000),PREOREPEARL(3000),
     &Y(3000),NOPARTINCA(1001),NOPARTINCWID(1001),NOPARTINCPEARL(1001),
     &NOPARTGBA(1001),NOPARTGBWID(1001),NOPARTGBPEARL(1001),
     &TIMEOFNUCL(1001),EXTTHICK(1001)
C
       PI=3.141592654
       IF (I2.EQ.1) THEN
C SET TOTAL ASSEMBLY VOLUME TO UNITY:
       VOLTOT=1.0
C ASSIGN NUMERICAL VALUES TO ARBITARY PLANES THAT ARE PARALLEL TO A
C PLANAR AUSTENITE GRAIN BOUNDARY. THREE SIZE STEPS ARE USED.
       INOSMA=1001
       INOMED=990
       INOLAR=900      
       IYPLANE=INOSMA+INOMED+INOLAR
       YEND=10000.0D-6
       DYSMA=0.01D-6
       DYMED=1.0D-6
       DYLAR=10.0D-6
C VARY HEIGHT OF PLANE TO BE CONSIDERED:
       DO 810 I810=1,IYPLANE,1
        IF (I810.LE.INOSMA) THEN
          Y(I810)=1.0*((I810-1)*DYSMA)
        ELSEIF ((I810.GT.INOSMA).AND.(I810.LE.(INOSMA+INOMED))) THEN
          Y(I810)=Y(INOSMA)+1.0*((I810-INOSMA)*DYMED)
        ELSE
          Y(I810)=Y(INOSMA+INOMED)+1.0*((I810-INOSMA-INOMED)*DYLAR)
        ENDIF
        IF (I810.EQ.INOSMA) THEN
         Y(I810)=Y(I810)
        ELSEIF (I810.EQ.(INOSMA+INOMED)) THEN
         Y(I810)=Y(I810)
        ELSEIF (I810.EQ.IYPLANE) THEN
         Y(I810)=Y(I810)
        ENDIF
810    CONTINUE
       IFLAG=0
       ENDIF
C STORE NUCLEATION TIME OF PARTICLES:
       TIMEOFNUCL(I2)=PRETIME
C CALCULATION OF THE INCREMENTS IN THE EXTENDED VOLUMES OF THE VARIOUS PHASES
C RANDOMLY NUCLEATED ON INCLUSIONS:
C STORE NO OF PARTICLES NUC AT 'START' OF THIS TIME INTERVAL
       NOPARTINCA(I2)=VIINCA*DTIME*VOLTOT
       NOPARTINCWID(I2)=VIINCWID*DTIME*VOLTOT
       NOPARTINCPEARL(I2)=VIINCPEARL*DTIME*VOLTOT        
       SUMINCA=0.0
       SUMINCWID=0.0
       SUMINCPEARL=0.0
       DO 610 I610=1,I2,1
C TIME ELAPSED SINCE THESE PARTICLES WERE NUCLEATED:
        TIMESNINC=PRETIME-TIMEOFNUCL(I610)
        IF (I610.EQ.I2) THEN
C PARTICLES JUST NUCLEATED:
         NEWCONINCA=NOPARTINCA(I610)*(4.0*PI/3.0)
     &   *(PARA3**3.0)
     &   *(DIFFUSY**1.5)*(DTIME**1.5)
         NEWCONINCWID=NOPARTINCWID(I610)
     &   *ARWID*(DTIME**3.0)*(GRWID**3.0)
         NEWCONINCPEARL=NOPARTINCPEARL(I610)*(4.0*PI/3.0)
     &   * (GRPEARL**3.0) * (DTIME**3.0)
         SUMINCA=SUMINCA+NEWCONINCA
         SUMINCWID=SUMINCWID+NEWCONINCWID
         SUMINCPEARL=SUMINCPEARL+NEWCONINCPEARL
        ELSE
C PARTICLES AREADY NUCLEATED:
         GROWCONINCA=NOPARTINCA(I610)*(4.0*PI/3.0)
     &   *(3.0/2.0)*(PARA3**3.0)
     &   *(DIFFUSY**1.5)*DTIME*(TIMESNINC**0.5)
         GROWCONINCWID=NOPARTINCWID(I610)
     &   *ARWID*3.0*(GRWID**3.0)
     &   *DTIME*(TIMESNINC**2.0)
         GROWCONINCPEARL=NOPARTINCPEARL(I610)*(4.0*PI/3.0)
     &   *(3.0)*(GRPEARL**3.0)*DTIME*(TIMESNINC**2.0)
         SUMINCA=SUMINCA+GROWCONINCA
         SUMINCWID=SUMINCWID+GROWCONINCWID
         SUMINCPEARL=SUMINCPEARL+GROWCONINCPEARL
        ENDIF                 
610    CONTINUE     
C INCREMENTS IN EXTENDED INCLUSION VOLUMES DURING THIS TIME INTERVAL:
       DVEXTAINC=SUMINCA
       DVEXTWIDINC=SUMINCWID
       DVEXTPEARLINC=SUMINCPEARL
C CALCULATION OF THE INCREMENTS IN THE EXTENDED VOLUMES OF THE VARIOUS
C BOUNDARY NUCLEATED PHASES:
C STORE THE NUMBER OF PARTICLES NUCLEATED ON BOUNDARIES DURING THIS TIME INTERVAL:
       NOPARTGBA(I2)=VIA*DTIME*OBOUND
       NOPARTGBWID(I2)=VIWID*DTIME*OBOUND
       NOPARTGBPEARL(I2)=VIPEARL*DTIME*OBOUND
C TO ALLOW FOR THE SECONDARY NUCLEATION OF WIDMANSTATTEN FERRITE AND PEARLITE
C CALCULATE THE MEAN EXTENDED THICKNESS AT THE PLANAR BOUNDARY CONSIDERED.
       TOTBOUNDEXTV=VAEXT+VWIDEXT+VPEARLEXT
       EXTTHICK(I2)=TOTBOUNDEXTV/OBOUND
       DO 20 I20=1,I2,1
C TIME ELAPSED SINCE THESE PARTICLES WERE NUCLEATED:
        TIMESN=PRETIME-TIMEOFNUCL(I20)
C DETERMINE THE HEIGHT OF THE VARIOUS EXTENDED PARTICLES:
        IF (TIMESN.LT.(0.000001*DTIME)) THEN
C PARTICLES JUST NUCLEATED:
         PREQA(I20)=0.0
         QA(I20)=PARA1*(DIFFUSY**0.5)*(DTIME**0.5)
         PREQWID(I20)=0.0
         QWID(I20)=(GRWID*DTIME)+EXTTHICK(I2)
         PREQPEARL(I20)=0.0
         QPEARL(I20)=(GRPEARL*DTIME)+EXTTHICK(I2)
        ELSE
C PARTICLES AREADY NUCLEATED:
         QAINCRE=0.5*PARA1*(DIFFUSY**0.5)
     &   *((TIMESN)**(-0.5))*DTIME
         PREQA(I20)=QA(I20)
         QA(I20)=QA(I20)+QAINCRE
         QWIDINCRE=GRWID*DTIME
         PREQWID(I20)=QWID(I20)
         QWID(I20)=QWID(I20)+QWIDINCRE
         QPEARLINCRE=GRPEARL*DTIME
         PREQPEARL(I20)=QPEARL(I20)
         QPEARL(I20)=QPEARL(I20)+QPEARLINCRE
        ENDIF
20     CONTINUE
C TEST THAT NO PARTICLES HAVE A HEIGHT GREATER THAN YEND:
       MAXHGTA=0.0
       MAXHGTWID=0.0
       MAXHGTPEARL=0.0
       DO 642 I642=1,I2,1
        IF (QA(I642).GT.MAXHGTA) THEN
         MAXHGTA=QA(I642)
        ENDIF
        IF (QPEARL(I642).GT.MAXHGTPEARL) THEN
         MAXHGTPEARL=QPEARL(I642)
        ENDIF
        IF (QWID(I642).GT.MAXHGTWID) THEN
         MAXHGTWID=QWID(I642)
        ENDIF
642    CONTINUE
       MAXHGT=MAXHGTWID      
       IF (MAXHGTPEARL.GT.MAXHGT) THEN
        MAXHGT=MAXHGTPEARL
       ENDIF
       IF (MAXHGTA.GT.MAXHGT) THEN
        MAXHGT=MAXHGTA
       ENDIF
       IF (MAXHGT.GT.YEND) THEN
        WRITE(*,*) "YEND TOO SMALL - ERROR !"
        PAUSE
       ENDIF
C COMPARE PARTICLES WITH THE VARIOUS ARBITRARY PLANES THAT ARE PARALLEL TO THE BOUNDARY
C TO DETERMINE THE INCREMENTS IN THE EXTENDED AREAS OF INTERSECTION:
       DO 10 I10=1,IYPLANE,1
C RESET TO ZERO:
        ALLA=0.0
        ALLWID=0.0
        ALLPEARL=0.0
C ONLY CONSIDER THOSE PLANES THAT INTERSECT THE EXTENDED PARTICLES:
        IF (MAXHGT.GT.Y(I10)) THEN
        DO 400 I400=1,I2,1
C TIME ELAPSED SINCE THESE PARTICLES WERE NUCLEATED:
         TIMESN=PRETIME-TIMEOFNUCL(I400)
C TEST IF THESE ALLOTRIMORPHIC FERRITE PARTICLES REACH THIS PLANE:
         IF (QA(I400).GE.Y(I10)) THEN
C FIRST TIME THESE PARTICLE HAVE REACHED THIS PLANE.
          IF ((PREQA(I400).LT.Y(I10)).OR.
     &    (TIMESN.LT.(0.000001*DTIME))) THEN
C ADD ALL OF CURRENT AREA OF THESE PARTICLES TO THE INCREMENT IN THE EXTENDED AREA:
           ALLA=ALLA+(NOPARTGBA(I400))*(PI*((AR*QA(I400))**2))
          ELSE
C ONLY NEED TO ACCOUNT FOR THE INCREASE IN THE EXTENDED AREA CONTRIBUTED BY THESE
C PARTICLES:
           ALLA=ALLA+
     &     NOPARTGBA(I400)*DTIME*((PARA1*AR)**2)*PI*DIFFUSY
          ENDIF
         ENDIF
C TEST IF THESE WIDMANSTATTEN FERRITE PARTICLES REACH THIS PLANE:
         IF (QWID(I400).GE.Y(I10)) THEN
C FIRST TIME THESE PARTICLE HAVE REACHED THIS PLANE.
          IF ((PREQWID(I400).LT.Y(I10)).OR.
     &    (TIMESN.LT.(0.000001*DTIME))) THEN
C ADD ALL OF CURRENT AREA OF THESE PARTICLES TO THE INCREMENT IN THE EXTENDED AREA:
           ALLWID=ALLWID
     &     +( (NOPARTGBWID(I400))
     &     *(ARWID*((QWID(I400)-EXTTHICK(I400))**2.0)) )
          ELSE
C ONLY NEED TO ACCOUNT FOR THE INCREASE IN THE EXTENDED AREA CONTRIBUTED BY THESE
C PARTICLES:
           ALLWID=ALLWID
     &     +( 2.0*(NOPARTGBWID(I400))*ARWID*(GRWID**2.0) 
     &     * TIMESN * DTIME )
          ENDIF
         ENDIF
C TEST IF THESE PEARLITE PARTICLES REACH THIS PLANE:
         IF (QPEARL(I400).GE.Y(I10)) THEN
C FIRST TIME THESE PARTICLE HAVE REACHED THIS PLANE.
          IF ((PREQPEARL(I400).LT.Y(I10)).OR.
     &    (TIMESN.LT.(0.000001*DTIME))) THEN
C ADD ALL OF CURRENT AREA OF THESE PARTICLES TO THE INCREMENT IN THE EXTENDED AREA:
           ALLPEARL=ALLPEARL
     &     + ( NOPARTGBPEARL(I400)*PI*
     &     (((QPEARL(I400)-EXTTHICK(I400)) * ARPEARL )**2.0) )
          ELSE
C ONLY NEED TO ACCOUNT FOR THE INCREASE IN THE EXTENDED AREA CONTRIBUTED BY THESE
C PARTICLES:
           ALLPEARL=ALLPEARL
     &     + ( 2.0*PI*NOPARTGBPEARL(I400)*(ARPEARL*GRPEARL**2.0) 
     &     * TIMESN * DTIME )
          ENDIF
         ENDIF
400     CONTINUE
        ELSE
C NO PARTICLES REACH THIS PLANE:
         ALLA=0.0
         ALLWID=0.0
         ALLPEARL=0.0    
        ENDIF
        DOEXA=ALLA
        DOEXWID=ALLWID
        DOEXPEARL=ALLPEARL
C CONVERT THESE INCREMENTS IN THE EXTENDED AREAS OF INTERSECTION TO THE CORRESPONDING
C INCREMENTS IN THE REAL AREAS OF INTERSECTION:
        IF (I2.EQ.1) THEN
C TRANSFORMATION JUST STARTED:
         DOREA=DOEXA
         DOREWID=DOEXWID
         DOREPEARL=DOEXPEARL
        ELSE
C CALCULATE THAT FRACTION OF THE AREA OF THIS PLANE REMAINING UNTRANSFORMED AT THIS TIME:
         IF((OREA(I10)+OREWID(I10)+OREPEARL(I10)).GT.OBOUND)
     &   THEN
          FRACOBUT=0.0
         ELSE
          FRACOBUT=(1.0-(OREA(I10)+OREWID(I10)+OREPEARL(I10))/OBOUND)
         ENDIF
C EXTENDED TO REAL AREA CONVERSION:
         DOREA=DOEXA*FRACOBUT
         DOREWID=DOEXWID*FRACOBUT
         DOREPEARL=DOEXPEARL*FRACOBUT
        ENDIF

C CORRECT CALCULATION AUG/2003 BY T. OKMURA
         DOTOTAL=DOREA+DOREWID+DOREPEARL

C SUM TO FIND TOTAL EXTENDED AREAS OF INTERSECTION:
        PREOREA(I10)=0.0
        PREOREWID(I10)=0.0
        PREOREPEARL(I10)=0.0
        IF ((OREA(I10)+OREWID(I10)+OREPEARL(I10)).LE.OBOUND) THEN
         PREOREA(I10)=OREA(I10)
         PREOREWID(I10)=OREWID(I10)
         PREOREPEARL(I10)=OREPEARL(I10)
        ENDIF
        OREA(I10)=OREA(I10)+DOREA
        OREWID(I10)=OREWID(I10)+DOREWID
        OREPEARL(I10)=OREPEARL(I10)+DOREPEARL

C ENSURE THAT TOTAL EXTENDED AREAS OF EACH PHASE AT EACH PLANE NEVER EXCEED OBOUND:
        IF ((OREA(I10)+OREWID(I10)+OREPEARL(I10)).GE.OBOUND) THEN
C         IF ((PREOREA(I10)+PREOREWID(I10)+PREOREPEARL(I10)).GT.0.0)
C     &   THEN   
C THIS CAUSES A PROBLEM WHEN THE FIRST NUCLEATION OF NEW PHASE
C AND SITE SATURATION OCCUR AT THE SAME TIME.
C CORRECTED SEP/2003 
C          PORA=PREOREA(I10)/
C     &    (PREOREA(I10)+PREOREPEARL(I10)+PREOREWID(I10))
C          PORWID=PREOREWID(I10)/
C     &    (PREOREA(I10)+PREOREPEARL(I10)+PREOREWID(I10))
C          PORPEARL=PREOREPEARL(I10)/
C     &    (PREOREA(I10)+PREOREPEARL(I10)+PREOREWID(I10))
          IF(DOTOTAL.EQ.0.0) THEN
           PORA=PREOREA(I10)/OBOUND
           PORWID=PREOREWID(I10)/OBOUND
           PORPEARL=PREOREPEARL(I10)/OBOUND
          ELSE
           PORA=PREOREA(I10)/OBOUND+DOREA*FRACOBUT/DOTOTAL
           PORWID=PREOREWID(I10)/OBOUND+DOREWID*FRACOBUT/DOTOTAL
           PORPEARL=PREOREPEARL(I10)/OBOUND+DOREPEARL*FRACOBUT/DOTOTAL
          ENDIF
C         ELSE
C          PORA=OREA(I10)/
C     &    (OREA(I10)+OREPEARL(I10)+OREWID(I10))
C          PORWID=OREWID(I10)/
C     &    (OREA(I10)+OREPEARL(I10)+OREWID(I10))
C          PORPEARL=OREPEARL(I10)/
C     &    (OREA(I10)+OREPEARL(I10)+OREWID(I10))
C         ENDIF
         OREA(I10)=PORA*OBOUND
         OREWID(I10)=PORWID*OBOUND
         OREPEARL(I10)=PORPEARL*OBOUND
        ENDIF
C NOTE THE TIME WHEN THE AUSTENITE GRAIN BOUNDARIES ARE FIRST SATURATED:
        IF ( (I10.EQ.1).AND.(IFLAG.EQ.0) ) THEN
         IF ( (OREA(I10)+OREWID(I10)+OREPEARL(I10))
     &   .GE. (0.99*OBOUND) ) THEN
          SATTIME=TIME
          IFLAG=1
         ENDIF
        ENDIF
10     CONTINUE
C CALCULATE THE TOTAL EXTENDED VOLUME OF EACH PHASE THAT ORIGINATES FROM THIS
C PLANAR BOUNDARY:
C RECONSTRUCTIVE PHASES - ALLOW FOR GROWTH ON BOTH SIDES OF THE BOUNDARY.
C DISPLACIVE - ALLOW FOR GROWTH ON ONE SIDE OF THE BOUNDARY.
C STORE PREVIOUS EXTENDED VOLUMES:
888    PREVAEXT=VAEXT
       PREVWIDEXT=VWIDEXT
       PREVPEARLEXT=VPEARLEXT
       VAEXT=( 2.0*0.5*DYSMA*( OREA(1)
     & +OREA(INOSMA) ) )
     & +( 2.0*0.5*DYMED*( OREA(INOSMA+1)
     & +OREA(INOSMA+INOMED) ) )
     & +( 2.0*0.5*DYLAR*( OREA(INOSMA+INOMED+1)
     & +OREA(IYPLANE) ) )
       VWIDEXT=( 1.0*0.5*DYSMA*( OREWID(1)
     & +OREWID(INOSMA) ) )
     & +( 1.0*0.5*DYMED*( OREWID(INOSMA+1)
     & +OREWID(INOSMA+INOMED) ) )
     & +( 1.0*0.5*DYLAR*( OREWID(INOSMA+INOMED+1)
     & +OREWID(IYPLANE) ) )
       VPEARLEXT=( 2.0*0.5*DYSMA*( OREPEARL(1)
     & +OREPEARL(INOSMA) ) )
     & +( 2.0*0.5*DYMED*( OREPEARL(INOSMA+1)
     & +OREPEARL(INOSMA+INOMED) ) )
     & +( 2.0*0.5*DYLAR*( OREPEARL(INOSMA+INOMED+1)
     & +OREPEARL(IYPLANE) ) )
       DO 500 I500=2,(IYPLANE-1),1
        IF ( (I500.GT.1).AND.(I500.LT.INOSMA) ) THEN
         VAEXT=VAEXT+(2.0*DYSMA*OREA(I500))
         VWIDEXT=VWIDEXT+(1.0*DYSMA*OREWID(I500))
         VPEARLEXT=VPEARLEXT+(2.0*DYSMA*OREPEARL(I500))
        ELSEIF ((I500.GT.(INOSMA+1)).AND.(I500.LT.(INOSMA+INOMED)))THEN
         VAEXT=VAEXT+(2.0*DYMED*OREA(I500))
         VWIDEXT=VWIDEXT+(1.0*DYMED*OREWID(I500))
         VPEARLEXT=VPEARLEXT+(2.0*DYMED*OREPEARL(I500))
        ELSEIF((I500.GT.(INOSMA+INOMED+1)).AND.(I500.LT.(IYPLANE)))THEN
         VAEXT=VAEXT+(2.0*DYLAR*OREA(I500))
         VWIDEXT=VWIDEXT+(1.0*DYLAR*OREWID(I500))
         VPEARLEXT=VPEARLEXT+(2.0*DYLAR*OREPEARL(I500))
        ENDIF
500    CONTINUE
C CALCULATE THE INCREMENTS IN THE VARIOUS EXTENDED VOLUMES DURING THE CURRENT TIME
C INTERVAL:
       IF (I2.EQ.1) THEN
        DVAEXT=VAEXT
        DVWIDEXT=VWIDEXT
        DVPEARLEXT=VPEARLEXT
       ELSE
        DVAEXT=VAEXT-PREVAEXT
        DVWIDEXT=VWIDEXT-PREVWIDEXT
        DVPEARLEXT=VPEARLEXT-PREVPEARLEXT
       ENDIF
C CONVERT THE INCREMENTS IN THE VARIOUS EXTENDED VOLUMES NUCLEATED RANDOMLY ON INCLUSIONS
C AND AT BOUNDARY SITES INTO THE CORRESPONDING REAL INCREMENTS.
C VOLUME FRACTION UNTRANSFORMED:
       FRACUT=1.0-
     & ( (VAREALINC+VAREAL+VPEARLREALINC+VPEARLREAL
     & +VWIDREALINC+VWIDREAL) / VOLTOT )
C CONVERT EXTENDED INCLUSION INCREMENTS TO REAL INCREMENTS:
       DVAREALINC=DVEXTAINC*FRACUT
       DVWIDREALINC=DVEXTWIDINC*FRACUT
       DVPEARLREALINC=DVEXTPEARLINC*FRACUT
C CONVERT EXTENDED BOUNDARY NUCLEATED INCREMENTS TO REAL INCREMENTS:
       DVAREAL=DVAEXT*FRACUT
       DVWIDREAL=DVWIDEXT*FRACUT
       DVPEARLREAL=DVPEARLEXT*FRACUT

C CONSIDER WHETHER TOTAL VOLUME IS MORE THAN ONE
C TO PREVENT IT FROM OVER LIMIT
C AUG/2003 BY T. Okumura
       DV=DVAREALINC+DVWIDREALINC+DVPEARLREALINC
     & +DVAREAL+DVWIDREAL+DVPEARLREAL
       IF (DV.GT.FRACUT) THEN
        DVAREALINC=DVAREALINC*FRACUT/DV
        DVWIDREALINC=DVWIDREALINC*FRACUT/DV
        DVPEARLREALINC=DVEPEARLREALINC*FRACUT/DV
        DVAREAL=DVAREAL*FRACUT/DV
        DVWIDREAL=DVWIDREAL*FRACUT/DV
        DVPEARLREAL=DVPEARLREAL*FRACUT/DV
       ENDIF
C SUM TO FIND TOTAL REAL INCLUSION NUCLEATED VOLUMES:
       VAREALINC=VAREALINC+DVAREALINC
       VWIDREALINC=VWIDREALINC+DVWIDREALINC
       VPEARLREALINC=VPEARLREALINC+DVPEARLREALINC
C SUM TO FIND TOTAL REAL BOUNDARY NUCLEATED VOLUMES:
       VAREAL=VAREAL+DVAREAL
       VWIDREAL=VWIDREAL+DVWIDREAL
       VPEARLREAL=VPEARLREAL+DVPEARLREAL
C TOTAL VOLUME OF ALLOTRIOMORPHIC AND IDIOMORPHIC FERRITE:
       VTOTAREAL=VAREAL+VAREALINC
C TOTAL VOLUME OF WIDMANSTATTEN FERRITE:
       VTOTWIDREAL=VWIDREAL+VWIDREALINC
C TOTAL VOLUME OF PEARLITE:
       VTOTPEARLREAL=VPEARLREAL+VPEARLREALINC
C TOTAL VOLUME OF ALL PHASES:              
       PREVTFERR=VTOTTRANS
       VTOTTRANS=VTOTWIDREAL+VTOTAREAL+VTOTPEARLREAL
       PERCENTRANS=(VTOTTRANS/EQVOLFRAC)*100.0
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE WHICH CONVERTS MOLE FRACTION TO WEIGHT %:
C INCLUDING COPPER JUNE/2003 BY T. OKUMURA

       SUBROUTINE MFTOWGT(CALLOY,XEQ,WCEQ)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z),INTEGER(I,J)
       DOUBLE PRECISION CALLOY(9)

       SI=CALLOY(2)   
       MN=CALLOY(3)
       NI=CALLOY(4)
       CR=CALLOY(6)
       MO=CALLOY(5)
       VAN=CALLOY(7)
       CU=CALLOY(8)
       C=CALLOY(1)
       FE=100.0D0-SI-MN-NI-MO-CR-VAN-C-CU
       BSI=28.0855
       BMN=54.938
       BNI=58.71
       BCR=51.996
       BMO=95.94
       BVAN=50.94
       BFE=55.847
       BCU=63.55
       BC=12.011
       ASI=SI/BSI    
       AMN=MN/BMN
       ANI=NI/BNI
       ACR=CR/BCR
       AMO=MO/BMO
       AVAN=VAN/BVAN
       ACU=CU/BCU
       AC=C/BC
       AFE=FE/BFE
       TA=ASI+AMN+ANI+ACR+AMO+AVAN+AC+AFE+ACU      
C MOLE FRACTION OF ELEMENTS IN AUSTENITE BEFORE TRANSFORMATION:
       XSI=ASI/TA    
       XMN=AMN/TA
       XNI=ANI/TA
       XCR=ACR/TA
       XMO=AMO/TA
       XVAN=AVAN/TA
       XCU=ACU/TA
       XC=AC/TA
       XFE=AFE/TA
       XFEAUS=(XFE+XC)-XEQ
C CONVERT PARAEQUILIBRIUM AUSTENITE COMP. TO WGT %:
       RSI=XSI*BSI    
       RMN=XMN*BMN
       RNI=XNI*BNI
       RCR=XCR*BCR
       RMO=XMO*BMO
       RVAN=XVAN*BVAN
       RCU=XCU*BCU
       RC=XEQ*BC
       RFE=XFEAUS*BFE
       TOTR=RSI+RMN+RNI+RCR+RMO+RVAN+RC+RFE+RCU
       WSI=100.0*RSI/TOTR   
       WMN=100.0*RMN/TOTR
       WNI=100.0*RNI/TOTR
       WCR=100.0*RCR/TOTR
       WMO=100.0*RMO/TOTR
       WVAN=100.0*RVAN/TOTR
       WCU=100.0*RCU/TOTR
       WCEQ=100.0*RC/TOTR
       WFEAUS=100.0*RFE/TOTR
       WTOT=(WSI+WMN+WNI+WCR+WMO+WVAN+WCEQ+WFEAUS+WCU)
       END

C SUBROUTINE BASED ON THE PROGRAM 'MUCG46.FOR' BY HKDH BHADESHIA WHICH IS USED TO CALCULATE
C THERMODYNAMIC DATA, A PHASE DIAGRAM AND A TTT DIAGRAM FOR THE ALLOY:
C INCLUDING COPPER JUNE/2003 BY T. OKUMURA
       SUBROUTINE MUCG(C,WS,BS,MS,T10,T20,W,STCTEMP,
     & STXEQ,STXEQ2)
       IMPLICIT REAL*8 (A-H,L-Z),INTEGER (I-K)
       DOUBLE PRECISION C(9),DT4(40),
     & DDFTO(40),DXQ(40),DFPRO(40),
     & BSH(40),BDIF(40),BTEM(40),STCTEMP(40),
     & STXEQ(40),STXEQ2(40)
C MODIFIED TO READ GMAX,SHEART,DIFFT,CTEMP INTO AN ARRAY
C FOR LATER USE.
       IMAX=0
C NO LONGER PRINT LOGO TO SAVE SPACE 
C       WRITE(*,244)
C       WRITE(*,446)
C       WRITE(*,245)
C       WRITE(*,246)
C       WRITE(*,247)
C       WRITE(*,248)
C       WRITE(*,446)
C       WRITE(*,259)
       WRITE(*,261) (C(J5),J5=1,8)
       CALL OMEGA(C,W,XBAR,T10,T20,0)
C       WRITE(*,260)
       WRITE(*,261) (C(J5),J5=1,8)
       FTO=-1.0
       X1=C(1)
       WRITE(*,446)
       XA=0.001D+00 
       R=8.31432D+00
       W1=48570.0D+00
       H=38575.0D+00
       S=13.48D+00
       WRITE(*,444) 
       XEQ2=0.15D+00
       STRAIN=50.00D+00
       XEQ=0.2D+00
       XTO=0.07D+00
       XTO400=0.06D+00
       X44=0.1D+00
       CONST=1.0D+00
       CORR=1.0D+00
       SLOPE=1.0D+00
       J8=0
       I77=0
       DO 9 J2=473,1173,20
        J8=J8+1
        XTO=XTO+0.0001D+00
        XTO400=XTO400+0.0001D+00
        X44=0.3*XEQ
        T=J2*1.0D+00
        CTEMP=T-273.00D+00
        IF (T .LE. 1000.0) GOTO 20
        H1=105525.0D+00
        S1=45.34521D+00
        GOTO 19
20      H1=111918.0D+00
        S1=51.44D+00
19      F=ENERGY(T,T10,T20)
        AJ=1.0D+00-DEXP(-W/(R*T))
        AJ1=1.0D+00-DEXP(-W1/(R*T))
51      TEQ=R*T*AFEG(XEQ,AJ)-F
        IF (DABS(TEQ) .LT. 1.0) GOTO 50
        ETEQ=DAFEG(XEQ,AJ)*R*T
        XEQ=XEQ-TEQ/ETEQ
        GOTO 51
50      TEQ2=R*T*AFEG(XEQ2,AJ)-F-STRAIN
        IF (DABS(TEQ2) .LT. 1.0) GOTO 52
        ETEQ2=DAFEG(XEQ2,AJ)*R*T
        XEQ2=XEQ2-TEQ2/ETEQ2
        GOTO 50
52      AEQ=CG(XEQ,T,W,R)
        AFEQ=AFEG(XEQ,AJ)
        IF (XEQ .LT. X1+0.0004) GOTO 2444
        A=CG(X1,T,W,R)
        AFE=AFEG(X1,AJ)
        FSON=R*T*(XA*(AEQ-A)+(1-XA)*(AFEQ-AFE))
        FPRO=R*T*(X1*(AEQ-A)+(1-X1)*(AFEQ-AFE))
        V14=WSFUN(T)
        IF (V14 .GE. 0.0) THEN
         X44=99999999999.99999
         GOTO 102
        ENDIF
103     A44=CG(X44,T,W,R)
        AFE44=AFEG(X44,AJ)
        F44=R*T*(X44*(AEQ-A44)+(1-X44)*(AFEQ-AFE44))
        F44=F44-V14
        IF (DABS(F44) .GE. 10.0) GOTO 101
        GOTO 102
101     DA44=DCG(X44,T,W,R)
        DAFE44=DAFEG(X44,AJ)
        DF441=R*T*(AEQ-X44*DA44-AFEQ+X44*DAFE44-DAFE44-A44+AFE44)
        X44=X44-F44/DF441
        GOTO 103
102     FPROA=FPRO*((XEQ-XA)/(XEQ-X1))
        CALL GMAAX(A1,A,W1,F,R,T,X,AFE,H1,S1)
        GMAX=R*T*(A1-A)
        IF (FTO .GE. 0.0) GOTO 22
        FTO=FTO1(H,S,X1,T,W,W1,H1,S1,F,AJ,AJ1)
        GOTO 21
22      FTO=0.0
21      CALL AXTO(H,S,H1,S1,XTO,T,W,W1,F,AJ,AJ1,0.0D+00,J98)
        CALL AXTO(H,S,H1,S1,XTO400,T,W,W1,F,AJ,AJ1,400.0D+00,J99)
        CALL TTTT(T,GMAX,SHEART,DIFFT)
        IF (X44 .EQ. 0.0) GOTO 453 
        GOTO 454
453     SHEART=1D+20
454     WRITE(*,8) FPRO,FPROA,GMAX,CTEMP,X,FSON,XEQ,XEQ2,FTO,XTO, 
     &  XTO400,SHEART,DIFFT
        I77=I77+1
        STXEQ(I77)=XEQ
        STXEQ2(I77)=XEQ2
        STCTEMP(I77)=CTEMP
        IMAX=IMAX+1
        BSH(IMAX)=SHEART
        BDIF(IMAX)=DIFFT
        BTEM(IMAX)=CTEMP
777     FORMAT(I5,3D12.2)
        DXQ(J8)=XEQ
        DDFTO(J8)=FTO
        DT4(J8)=CTEMP
        DFPRO(J8)=FPRO
9      CONTINUE
2444   J8=J8-1
       WRITE(*,446)
       CALL ANALY(J8,10,CONST,SLOPE,CORR,DT4,DDFTO,0)
       BS=(-400.0-CONST)/SLOPE
       CALL WSTINE(DFPRO,WS,WS1,J8)
C ALLOW FOR LARGE CARBON CONCENTRATIONS (BHADESHAI, METAL SCIENCE, VOL.15, 1981,
C P.175-177):
       XMS=X1
       IF(XMS .GT. 0.0594D+00)XMS=0.0594D+00
       MS=(-1086.9D+00 + 2941.8D+00*XMS - 9.6016D+05*XMS*XMS
     & + 2.5382D+07*XMS*XMS*XMS - 1.8488D+08*XMS*XMS*XMS*XMS
     & - CONST)/SLOPE    
       WRITE(*,9992)WS1,WS
       IF (WS .LT. BS) THEN
        BS=WS
        WRITE(*,9998)BS
       ELSE
        WRITE(*,9988)BS
       ENDIF
       WRITE(*,9994)MS
       WRITE(*,446)
244    FORMAT(////)
245    FORMAT(//,32X,'PREDICTED ALLOY THERMODYNAMICS & TTT DIAGRAM')
246    FORMAT(/,28X,'AS MUCG46.FOR INCLUDING COPPER')
247    FORMAT(/,44X,'BY HKDH BHADESHIA')
248    FORMAT(/,41X,'UNIVERSITY OF CAMBRIDGE',//)
446    FORMAT('--------------------------------------------------',
     & '-------------------------------',
     & '------------------------------')
8      FORMAT (1H ,F7.0,2F7.0,F6.0,D10.2,F8.0,
     & 2F9.4,F7.0,F8.4,1H ,F8.4,1H ,D11.2,D11.2)
444    FORMAT(2X,'FPRO',3X,'FPROA',2X,'GMAX',4X,'CTEMP',1X,
     & 'XNUCLEUS',2X,'FSON',5X,'XEQ',6X,'XEQ50',
     & 2X,'FTO',5X,'XTO',6X,'XTO400',4X,
     & 'SHEART',5X,'DIFFT')
9992   FORMAT('   WIDMANSTATTEN START TEMPERATURES ',F7.0,' & ',F7.0)
9994   FORMAT('   MARTENSITE START TEMPERATURE',F7.0,' C')
9988   FORMAT('   GROWTH LIMITED BAINITE START TEMPERATURE',F7.0,' C')
9998   FORMAT('   NUCLEATION LIMITED BAINITE START TEMPERATURE
     & ',F7.0,' C')
259    FORMAT('    ALLOY COMPOSITION IN WT % :')
260    FORMAT('    ALLOY COMPOSITION IN MOLE FRACTION :')
261    FORMAT (6H    C=,F8.4,6H   SI=,F8.4,6H   MN=,F8.4,
     & 6H   NI=,F8.4,6H   MO=,F8.4,6H   CR=,F8.4,6H    V=,F8.4,
     & 6H   CU=,F8.4)
26     FORMAT (7F6.2,I5)
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE TO CALCULATE INTERFACE COMPOSITIONS (GAMMA/GAMMA+ALPHA) ASSUMING NEGLIGIBLE
C PARTITIONING LOCAL EQUILIBRIUM:
       SUBROUTINE NPGA0(T,X,Y,IM)
       IMPLICIT REAL*8 (A-H,K-Z)
       DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),K(8),
     & XX(3),G(8),G2(8),G3(8),G4(8),GM(8),G0(8),
     & LFX(4,8),AC(8),E(8,8),EA(8,8),FUN(3)
       COMMON/TRANS/C0,W0,K
       COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC,
     & GFE3C,G3,G4,WW,E,EA,G,G0
       R=8.314D+00
       XGA=C0(1)+0.01D+00
       ITER=0
       ACC=1.0D-05
       DX1=1.0D-10
200    XX(1)=XGA-DX1
       XX(2)=XGA+DX1
       XX(3)=XGA
       ITER=ITER+1
       IF(ITER .GT. 10) THEN
        ACC=ACC*10
        ITER=0
       ENDIF      
       DO 220 I=1,3
        X1=XX(I)
        A1=DEXP(G0(1)/R/T+E(1,1)*X1)/
     &  (1.0D+00+EA(1,1)*X1*DEXP(G0(1)/R/T))
        A2=DEXP(G0(IM)/R/T+E(1,IM)*X1)/
     &  (1.0D+00+E(1,IM)*X1*DEXP(G0(1)/R/T))
        Y1=A1*X1
        Y2=C0(IM)
        X2=Y2/A2
        X0=1.0D+00-X1-X2
        Y0=1.0D+00-Y1-Y2
        FFG=DLOG(X0)-1.0/2.0*E(1,1)*X1*X1-1.0/2.0*E(IM,IM)*X2*X2
     &  -E(1,IM)*X1*X2
        FFA=DLOG(Y0)-1.0/2.0*EA(1,1)*Y1*Y1-1.0/2.0*EA(IM,IM)*Y2*Y2
     &  -E(1,IM)*Y1*Y2
        FUN(I)=G4(8)/R/T+FFG-FFA
220    CONTINUE    
       IF (ABS(FUN(3)) .LT. ACC) GOTO 100
       DUMMY=XGA-2.0*DX1*FUN(3)/(FUN(2)-FUN(1))
       IF (DUMMY .LT. 0.0D+00) THEN
        DUMMY=XGA/2.0D+00
       ENDIF
       IF (DUMMY .GT. 0.25D+00) THEN
        DUMMY=(0.25D+00+XGA)/2.0
       ENDIF
       XGA=DUMMY
       GOTO 200
100    X(1)=X1
       X(IM)=X2
       Y(1)=Y1
       Y(IM)=Y2
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE TO CALCULATE THE EQUILIBRIUM CHEMISTRY AT THE AUSTENITE / AUSTENITE+CEMENTITE
C INTERFACE:
       SUBROUTINE NPGC0(T,X,Y,IM)
       IMPLICIT REAL*8 (A-H,K-Z)
       DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),
     & K(8),XX(3),FUN(3),G0(8),
     & G(8),G2(8),G3(8),G4(8),GM(8),MX(8),MM3C(8),
     & LFX(4,8),AC(8),E(8,8),EA(8,8)
       INTEGER ITER
       COMMON/TRANS/C0,W0,K
       COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC,
     & GFE3C,G3,G4,WW,E,EA,G,G0
       R=8.314D+00
C       XGC=C0(IM)*0.99D+00
       DUMMY=1.0D-6
       ITER=0
C MODIFY THE WAY OF CALCULATION
C AUG/2003 BY T. OKUMURA
C100    XX(1)=XGC-DUMMY
C       XX(2)=XGC+DUMMY
C       XX(3)=XGC
       XX(1)=1.0D-6
       XX(3)=0.9D0
 100   XX(2)=0.5*(XX(1)+XX(3))
       DO 10 I=1,3
        X1=XX(I)
        FUN(I)=0.0D+00
        B=DEXP((GFE3C-G(IM)-WW(IM))/R/T+E(1,IM)*X1)
        Y2=C0(IM)*4.0/3.0
        X2=Y2/B
        X8=1.0D+00-X1-X2
        Y8=1.0D+00-Y2
        IF (X(8) .LT. 0.0D+00) THEN
         WRITE(6,*) 'X8,X1=',X8,X1
        ENDIF
        MFE3C=R*T*DLOG(Y8)
        MFE=R*T*DLOG(X8)-R*T/2.0*E(1,1)*X1*X1
        MC=R*T/3.0*DLOG(X1)+R*T/3.0*E(1,1)*X1
        MFE3C=MFE3C+(1-Y8)*WW(IM)*Y2
        MFE=MFE-R*T/2.0*E(IM,IM)*X2*X2-R*T*X1*E(1,IM)*X2
        MC=MC+R*T/3.0*E(1,IM)*X2
        MX(IM)= R*T*DLOG(X2)+R*T*(E(1,IM)*X1+E(IM,IM)*X2)
        MM3C(IM)= R*T*DLOG(Y2)+Y8*WW(IM)
        MM3C(IM)=MM3C(IM)-Y8*WW(IM)*Y2
        FUN(I)=GFE3C+MFE3C-MFE-MC
10     CONTINUE
       IF(ABS(FUN(3)) .LT. 1.0D-02) GOTO 20
C SEE ABOVE
C       DUMMY1=XGC-2.0*DUMMY*FUN(3)/(FUN(2)-FUN(1))
C       IF (DUMMY1 .LT. 0.0D+00) THEN
C        DUMMY1=XGC/2.0D+00
       IF(FUN(2).LT.0.0) THEN
        XX(3)=XX(2)
       ELSE
        XX(1)=XX(2)
       ENDIF
        ITER=ITER+1
        IF(ITER.GT.10000) THEN
        WRITE(*,*) 'NO ANSWERE IN NPGC'
        STOP
        ENDIF
C       XGC=DUMMY1
       GOTO 100
20     X(1)=X1
       X(IM)=X2
       Y(1)=0.25D+00
       Y(IM)=3.0D+00/4.0D+00*Y2
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C CARBON-CARBON INTERACTION ENERGY (J/MOL) IN AUSTENITE:
C C IS A MATRIX OF COMPOSITION, INPUT IN WT % AND CHANGED INTO MOLE FRACTION ON EXIT.
C T10 IS DELTA(TMAG - TNONMAG).
C T20 IS DELTA TMAG.
C W IS THE CARBON-CARBON INTERACTION ENERGY IN AUSTENITE.
C XBAR IS RETURNED AS THE MOLE FRACTION OF CARBON.
C
C INCLUDING COPPER (JUN/2003) T.Okumura

       SUBROUTINE OMEGA(C,W,XBAR,T10,T20,J)
       DOUBLE PRECISION C(9),W,P(9),B1,B2,Y(9),T10,T20,B3,XBAR
       INTEGER U,J
       B3=0.0D+00
       IF(J .EQ. 1) GOTO 1
       C(9)=100.0-(C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8))
       C(9)=C(9)/55.84D+00
       C(1)=C(1)/12.0115D+00
       C(2)=C(2)/28.09D+00
       C(3)=C(3)/54.94D+00
       C(4)=C(4)/58.71D+00
       C(5)=C(5)/95.94D+00
       C(6)=C(6)/52.0D+00
       C(7)=C(7)/50.94D+00
       C(8)=C(8)/63.55D+00
1      B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)+C(9)
C DERIVE RATIOS OF SUBSTITUTIONAL SOLUTE TO IRON ATOMS
       DO 2 U=2,8
        Y(U)=C(U)/C(9)
2      CONTINUE
C DERIVE MOLE FRACTIONS OF ALL CONCENTRATIONS
       DO 3 U=1,9
        C(U)=C(U)/B1
3      CONTINUE
C SETS XBAR TO ZERO IF LESS THAN A CERTAIN VALUE
       XBAR=C(1)
       XBAR=DINT(10000.0D+00*XBAR)
       XBAR=XBAR/10000
       B2=0.0D+00

C EVALUATE T10=DELTA (TMAG-TNONMAG) AARONSON, DOMIAN, POUND, TMS AIME 236 (1966) 769.
C CORRECT VALUES FOR COPPER SEP/2003
       T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)
     & +Y(8)*16
C     & +Y(8)*7
C EVALUATE T20=DELTA T MAG AARONSON, DOMIAN, POUND, TMS AIME 236 (1966) 769. 
       T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)-44*Y(7)
     & +4.5*Y(8)
C     & -4.5*Y(8)
C POLYNOMIALS REPRESENTING CARBON-CARBON INTERACTION ENERGY (J/MOL)
       P(2)=2013.0341+763.8167*C(2)+45802.87*C(2)**2-280061.63*C(2)**3
     & +3.864D+06*C(2)**4-2.4233D+07*C(2)**5+6.9547D+07*C(2)**6
       P(3)=2012.067-1764.095*C(3)+6287.52*C(3)**2-21647.96*C(3)**3-
     & 2.0119D+06*C(3)**4+3.1716D+07*C(3)**5-1.3885D+08*C(3)**6
       P(4)=2006.8017+2330.2424*C(4)-54915.32*C(4)**2+1.6216D+06*C(4)**3
     & -2.4968D+07*C(4)**4+1.8838D+08*C(4)**5-5.5531D+08*C(4)**6
       P(5)=2006.834-2997.314*C(5)-37906.61*C(5)**2+1.0328D+06*C(5)**3
     & -1.3306D+07*C(5)**4+8.411D+07*C(5)**5-2.0826D+08*C(5)**6
       P(6)=2012.367-9224.2655*C(6)+33657.8*C(6)**2-566827.83*C(6)**3
     & +8.5676D+06*C(6)**4-6.7482D+07*C(6)**5 +2.0837D+08*C(6)**6
       P(7)=2011.9996-6247.9118*C(7)+5411.7566*C(7)**2
     & +250118.1085*C(7)**3-4.1676D+06*C(7)**4
       P(8)=2011.0
C EVALUATE CARBON-CARBON INTERACTION ENERGY FOR MULTICOMPONENT SYSTEM ACCORDING TO: K R
C KINSMAN AND H I AARONSON, "TRANSFORMATION AND HARDENABILITY IN STEELS", CLIMAX
C MOLYBDENUM, ANN ARBOR, MI, USA, (1967) 33-38:
       DO 4 U=2,8
        B3=B3+P(U)*Y(U)
        B2=B2+Y(U)
4      CONTINUE
       IF (B2 .LT. 1D-40) THEN
        W=8054.0
       ELSE
        W=(B3/B2)*4.187
       ENDIF
       IF (DABS(B3).LT.1D-40) THEN
        W=8054
       ENDIF
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C MODEL USED TO CALCULATE THE THERMODYNAMICS AND KINETICS OF THE PEARLITE TRANSFORMATION:
       SUBROUTINE PEARL(W1,W2,W3,W4,W5,W6,W7,TI,ISAMPLE,IPARA,
     & PX1GC,PX2GC,PX1CG,PX2CG, PX1GA,PX2GA,PX1AG,PX2AG,VEL1,
     & X1GC,X2GC,X1CG,X2CG, X1GA,X2GA,X1AG,X2AG, 
     & VEL2,SC,S,JPEARL)
C BY M. TAKAHASHI, DEPARTMENT OF MATERIALS, UNIVERSITY OF CAMBRIDGE, UK, 1992.
C NOTE THAT GAMMA IS AUSTENITE, ALPHA IS FERRITE AND THETA IS CEMENTITE.
C THE GAMMA/GAMMA+ALPHA AND GAMMA/GAMMA+THETA INTERFACE COMPOSITIONS ARE CALCULATED USING
C THE NEWTON-RAPHSON METHOD ASSUMING THE HILLERT-STAFFANSON  REGULAR SOLUTION MODEL IS
C APPLICABLE.
C REFERENCE
C K. HASHIGUCHI ET AL., CALPHAD VOL.8, NO.2, 173-186, 1984
C J.S.KIRKALDY ET AL., HARDENABILITY CONCEPTS WITH APPLICATIONS TO STEEL, 82-125
C J.B. GILMOUR ET AL., MET. TRANS. VOL.3, 1455-64, 1972
C IPARA=1 IF ONLY PARAEQUILIBRIUM RATE OF GROWTH REQUIRED
C IPARA=0 SUBSTITUTIONAL SOLUTE DIFFUSION WITH GRAIN BOUNDARY 
       IMPLICIT REAL*8 (A-H,K-Z)
       DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),K(8),G0(8),
     & G(8),G2(8),G3(8),G4(8),GM(8),LFX(4,8),AC(8),E(8,8),EA(8,8)
       COMMON/TRANS/C0,W0,K
       COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC
     &           ,GFE3C,G3,G4,WW,E,EA,G,G0      
       IS=0
10     IS=IS+1
       IF (IS .GT. ISAMPLE) GOTO 100
C  1  2  3  4  5  6  7  8
C  C  SI MN NI CR MO CU FE
       W0(1)=W1
       W0(2)=W2
       W0(3)=W3
       W0(4)=W4
       W0(5)=W5
       W0(6)=W6
       W0(7)=W7   
       DT=10.0D+00
       TF=TI
       CALL CONV(1,W0,C0)
199    FORMAT(1H ,'      C      SI      MN      NI',
     & '      CR       MO      CU  ')
198    FORMAT(1H ,7F9.5)  
       CALL PEARLOMEGA(W,T10,T20)
C COMPUTE THE MOLE FRACTION OF IRON BY DIFFERENCE:
       CC=0.0D+00
       DO 11 I=1,7
        CC=CC+C0(I)
11     CONTINUE
       C0(8)=1.0D+00-CC
C CALCULATE THE SUBSTITUTIONAL SOLUTE TO IRON ATOM RATIOS:
       KK=0.0D+00
       DO 12 I=2,7
        K(I)=C0(I)/C0(8)
        KK=KK+K(I)
        IF (C0(I) .EQ. 0.0D+00) GOTO 12
         IM=I
C         IF (IM .GT. 6) IM=6
12     CONTINUE
       YY=0.0D+00
       DO 13 I=2,7
        Y(I)=K(I)/(1.0D+00+KK)
        YY=YY+Y(I)
13     CONTINUE
       Y(8)=1.0D+00-YY
       CTEMP=TI+DT
20     CTEMP=CTEMP-DT
       T=CTEMP+273.0D+00
       IF (CTEMP .LT. TF) GOTO 100
       IFLAGF=0
       IFLAGC=0
C THERMODYNAMIC PARAMETERS IN THE ORDER; SI,MN,NI,CR,MO,CU (DATA DUE TO UHRENIUS AND
C KIRKALDY ET AL.):

C DG:GIBBS FREE ENERGY CHNAGE FROM GRAPHITE TO CARBON IN GAMMA
       DG=46150.0D+00/3.0-19.205D+00*T/3.0

C GFE3C:GIBBS FREE ENERGY CHANGE FROM IRON AND CARBON IN GAMMA TO CEMENTITE
       GFE3C=1.332D+04-64.718*T+7.481*T*DLOG(T)-DG

C SFG: MAGNETIC COMPONENT OF ENTOROPY CHANGE OF ALPHA (T<1075K)
       SFG=-11.906D-04*T+8.272D-06*T*T-15.079D-09*T*T*T
     & +12.857D-12*T*T*T*T

C G0: GIBBS FREE ENERGY CHANGE FROM ALPHA TO GAMMA
       G0(1)=-65562.0D+00+32.949*T
       G0(2)=7087-7.125*T  
       G0(3)=-20520.0D+00+4.086D+00*T+1500.0D+00*SFG
       G0(4)=12950+5.02*T+383*SFG
       G0(5)=-1534.0D+00-19.472*T+2.749*T*DLOG(T)
       G0(6)=310-0.285*T+400*SFG
       G0(7)=-11966+1.602*T+1000*SFG

C G: GIBBS FREE ENERGY CHANGE FROM GAMMA TO CARBIDE
       G(2)=28535.0D+00-DG
       G(3)=-14263.0D+00+10.0D+00*T-DG
       G(4)=20338.0D+00-2.368D+00*T-DG
       G(5)=-24418.0D+00+16.61D+00*T-2.749D+00*T*DLOG(T)-DG
       G(6)=-19644.0D+00-0.628*T-DG
       G(7)=28535.0D+00-DG
       G(8)=1.3320-64.718+7.481*T*DLOG(T)

C CARBON-CARBON AND CARBON-SUBSTITUTIONAL SOLUTE INTERACTION PARAMETERS (DUE TO KIRKALDY
C ET AL.):
C INTERACTION COEFFICIENTS OF CARBON AND SOLUTE

C     REVISED Metal Science, Vol. 14, 1980, pp. 230-232
C       E(1,1)=4.7859D+00+5066.0D+00/T
       E(1,1)=8910.0D+00/T
       E(1,2)=14795.0D+00/T
       E(1,3)=-4811.0D+00/T
       E(1,4)=-2.2D+00+7600.0D+00/T
       E(1,5)=24.4-38400.0D+00/T
       E(1,6)=3.855D+00-17870.0D+00/T
       E(1,7)=4200.0D+00/T

C SELF-INTERACTION COEFFICIENTS OF SOLUTES IN GAMMA
       E(2,2)=26048.0D+00/T
       E(3,3)=2.406D+00-175.6D+00/T
       E(4,4)=-721.7D+00/T
       E(5,5)=7.655D+00-3154.0D+00/T-0.661D+00*DLOG(T)
       E(6,6)=-2330.0D+00/T
       E(7,7)=-0.161D+00-7834.0D+00/T

C SELF-INTERACTION COEFFICIENTTS OF SOLUTES IN ALPHA
       EA(1,1)=1.3D+00
       EA(2,2)=-13.31D+00+44088.0/T
       EA(3,3)=3.082D+00-4679.0/T+1509.8*SFG/T
       EA(4,4)=2.041D+00-2478.0/T+385.5*SFG/T
       EA(5,5)=2.819D+00-6039.0/T
       EA(6,6)=-0.219D+00-4772.0/T+402.6*SFG/T
       EA(7,7)=0.634D+00-11270.0/T+1006.5*SFG/T

C INTERACTIN COEFFICIENTS BETWEEN FE AND ALLOY IN CEMENTITE
       WW(2)=0.0D+00
       WW(3)=8351.0D+00-15.188D+00*T
       WW(4)=0.0D+00
       WW(5)=1791.0D+00
       WW(6)=0.0D+00
       WW(7)=0.0D+00

C UHRENIUS 'HARDENABILITY CONCEPT WITH APPLICATIONS TO
C STEEL' 1978 P28-81

C FREE ENERGY CHANGE FROM FE-CARBIDE+ALLOY TO 
C FE + ALLOY-CARBIDE BETWEEN STANDARD STATES
C IN GAMMA
       G2(2)=123000D+00
       G2(3)=-48500D+00
       G2(4)=46000D+00
       G2(5)=-251160D+00+118.0D+00*T
       G2(6)=-267200D+00
       G2(7)=-46000.0D+00+55.0D+00*T

C FREE ENERGY CHANGE FROM FE-CARBIDE+ALLOY TO 
C FE + ALLOY-CARBIDE BETWEEN STANDARD STATES
C IN FERRITE
       G3(2)=404180.0D+00
       G3(3)=-145500.0D+00
       G3(4)=138000.0D+00
       G3(5)=-153640.0D+00+38.860*T
       G3(6)=-267200.0D+00
       G3(7)=-46000.0D+00+55.0*T

C PARTIAL FREE ENERGY CHANGE OF A ELEMENT
C ACCOMPANYING WITH FERRITE TO GAMMA TRANSFORMATION
       G4(1)=-65563.0D+00+23.815*T
       G4(2)=-64818.0D+00+38.543*T
       G4(3)=-1800.0D+00+1.276*T
       G4(4)=-5650.0D+00-3.35*T
       G4(5)=10460.0D+00+0.628*T
       G4(6)=10460.0D+00+0.628*T
       G4(7)=-6276.0D+00+3.347*T
       G4(8)=(-1.0D+00)*PEARLENERGY(T,T10,T20)

C INTERACTION PARAMETER BETWEEN IRON AND AN ALLOY ELEMENT
C WHEN THE METAL LATTICE IS COMPLETELY FILLED WITH CARBON
C ATOMS
C STANDARD STATE IN GAMMA
       LFX(1,2)=-108280.0D+00
       LFX(1,3)=730.0D+00-10.0D+00*T
       LFX(1,4)=-14600.0D+00
       LFX(1,5)=13110.0D+00-31.82D+00*T+2.748D+00*T*DLOG(T)
       LFX(1,6)=9686.0D+00
       LFX(1,7)=49752.0D+00-9.431D+00*T
C 
       LFX(2,2)=0.0D+00
       LFX(2,3)=0.0D+00
       LFX(2,4)=8800.0D+00
       LFX(2,5)=0.0D+00
       LFX(2,6)=0.0D+00
       LFX(2,7)=-8594.0D+00+5.05D+00*T
C
       LFX(3,2)=-177100.0D+00+30.02*T
       LFX(3,3)=19450.0D+00-12.81*T-1500.0*SFG
       LFX(3,4)=-7300.0D+00-8.73*T-383.0*SFG
       LFX(3,5)=25104.0D+00-11.720*T
       LFX(3,6)=19836.0D+00+0.912*T-400.0*SFG
       LFX(3,7)=46848.0D+00-2.636*T-1000.0*SFG
C
       LFX(4,2)=-3085.0D+00+12.65*T
       LFX(4,3)=0.0D+00
       LFX(4,4)=8800.0D+00
       LFX(4,5)=0.0D+00
       LFX(4,6)=0.0D+00
       LFX(4,7)=0.0D+00

C INTERACTION BETWEEN CARBON AND VACANCY
       LCV=-21058.0D+00-11.581D+00*T

C FREE ENERGY CHANGE FROM ALLOY AND GRAPHITE
C TO ALLOY CARBIDE BETWEEN STANDARD STATES
       GM(2)=0.0D+00
       GM(3)=-13532.0D+00
       GM(4)=14540.0D+00-2.367D+00*T
       GM(5)=-850.0D+00-14.58D+00*T
       GM(6)=502.0D+00
       GM(7)=28535.0D+00

C SEE WW( )
       AC(2)=0.0D+00
       AC(3)=8350.0D+00-15.2D+00*T
       AC(4)=0.0D+00
       AC(5)=1790.0D+00
       AC(6)=0.0D+00
       AC(7)=0.0D+00

C SEE GFE3C
       GFEC=1.332D+04-64.718*T+7.481*T*DLOG(T)-DG

C SEE DG
       DDG=46150.0D+00-19.221D+00*T

C CALCULATION OF PARAEQUILIBRIUM COMPOSITIONS (GAMMA/GAMMA+ALPHA):
       CALL PEQGA0(T,X,Y,IM)
C MOLE FRACTION OF CARBON IN AUSTENITE IN PARAEQUILIBRIUM WITH FERRITE:
       PX1GA=X(1)
C MOLE FRACTION OF SUBSTITUTIONAL SOLUTE IN AUSTENITE IN PARAEQUILIBRIUM WITH FERRITE:
       PX2GA=X(IM)
C MOLE FRACTION OF CARBON IN FERRITE IN PARAEQUILIBRIUM WITH AUSTENITE:
       PX1AG=Y(1)
C MOLE FRACATION OF SUBSTITUTIONAL SOLUTE IN FERRITE IN PARAEQUILIBRIUM WITH FERRITE:
       PX2AG=Y(IM)
C CALCULATION OF PARAEQUILIBRIUM COMPOSITIONS (GAMMA/GAMMA+THETA)
       CALL PEQGC0(T,X,Y,IM)
C MOLE FRACTION OF CARBON IN AUSTENITE IN PARAEQUILIBRIUM WITH CEMENTITE
       PX1GC=X(1)
C MOLE FRACTION OF SUSBTITUTIONAL SOLUTE IN AUSTENITE IN PARAEQ. WITH CEMENTITE
       PX2GC=X(IM)
C MOLE FRACTION CARBON IN CEMENTITE PARAEQUILIBRIUM WITH AUSTENITE
       PX1CG=0.25D+00
C MOLE FRACTION SUBSTITUTIONAL SOLUTE IN CEMENTITE IN PARAEQ. WITH AUSTENITE
       PX2CG=Y(IM)
C CALCULATION OF VOLUME DIFFUSION CONTROLLED GROWTH RATE OF PEARLITE:
C SC AND S ARE THE EMPIRICAL CRITICAL AND ACTUAL INTERLAMELLAR SPACINGS.
       SC=2.0*600.0*1000.0/6.09D09/(1000.0D+00-(CTEMP+273.15))
       S=1.0D-04/(127.351D+00-0.17368D+00*CTEMP-4.9195*W0(3)
     & +1.7868*W0(5))
       CCC=(PX1GA+PX1GC)/2.0
       D=DIFF(T,CCC,CCC*1.001,W)
       IF(JPEARL .EQ. 1) THEN
        VEL1=VD(PX1GA,PX1GC,PX1AG,SC,D,S)
       ELSE
        VEL1=0.0D+00
       ENDIF
99     FORMAT(1H ,'AT TEMP=',F6.1,' CGC=',F8.6,' XGC=',
     & F8.6,' CCG=',F8.6,' XCG=',F8.6)
69     FORMAT(1H ,'               CGA=',F8.6,' XGA=',
     & F8.6,' CAG=',F8.6,' XAG=',F8.6)
68     FORMAT(1H ,'  V(BULK DIFF OF C) / CM/SEC=',D12.5,
     & '    DIFF. COEF=',D10.4)
C CALCULATION OF THE INTERFACE COMPOSITIONS (GAMMA/GAMMA+ALPHA):
C CALCULATION OF ACTIVITY OF CARBON IN GAMMA:
       ACTIV=ACG0(T,C0(1),C0(IM),E(1,1),E(1,IM))
       CALL NPGA0(T,X,Y,IM)
       X1GANP=X(1)
       X2GANP=X(IM)
       X1AGNP=Y(1)
       X2AGNP=Y(IM)
       X2TRANS=X2AC0(T,X1GANP,ACTIV,E(1,1),E(1,IM))
       IF (X2GANP .LT. X2TRANS) THEN
        IFLAGF=1
        X1GA=X1GANP
        X2GA=X2GANP
        X1AG=X1AGNP
        X2AG=X2AGNP
       ELSE
        CALL EQGA0(T,X,Y,ACTIV,IM)
        X1GA=X(1)
        X2GA=X(IM)
        X1AG=Y(1)
        X2AG=Y(IM)
       ENDIF
C CALCULATION OF THE INTERFACE COMPOSITIONS (GAMMA/GAMMA+THETA):
       CALL NPGC0(T,X,Y,IM)
       X1GCNP=X(1)
       X2GCNP=X(IM)
        X1CGNP=0.25D+00
       X2CGNP=Y(IM)
       X2TRANS=X2AC0(T,X1GCNP,ACTIV,E(1,1),E(1,IM))
       IF(X2GCNP .GT. X2TRANS) THEN
        IFLAGC=1
        X1GC=X1GCNP
        X2GC=X2GCNP
        X1CG=X1CGNP
        X2CG=X2CGNP
       ELSE
        CALL EQGC0(T,X,Y,ACTIV,IM)
        X1GC=X(1)
        X2GC=X(IM)
        X1CG=0.25D+00
        X2CG=Y(IM)
       ENDIF
C CALCULATION OF THE BOUNDARY DIFFUSION CONTROLLED GROWTH RATE OF PEARLITE:
       IF(JPEARL .EQ. 1) THEN
        VEL2=VBD(T,X2GA,X2GC,SC,C0(IM),S,IM)
       ELSE
        VEL2=0.0D+00
       ENDIF
50     CONTINUE
98     FORMAT(1H ,'AT TEMP=',F6.1,' CGC=',F8.6,' XGC=',
     & F8.6,' CCG=',F8.6,' XCG=',F8.6)
67     FORMAT(1H ,'               CGA=',F8.6,' XGA=',
     & F8.6,' CAG=',F8.6,' XAG=',F8.6)
66     FORMAT(1H ,'  V(BOUNDARY DIFF OF X) /CM/SEC=',D12.5)
       GOTO 20
100    RETURN
       END
C
C-------------------------------------------------------------------------------
C

C FUNCTION USED TO IN THE CALCULATION OF PEARLITE THERMODYNAMICS:
       DOUBLE PRECISION FUNCTION PEARLENERGY(T,T10,T20)
       DOUBLE PRECISION T,T10,T20,F,T7
       T7=T-1.0D+02*T20
       IF (T7 .LT. 3.0D+02) GOTO 1
       IF (T7 .LT. 7.0D+02) GOTO 2
       IF (T7 .LT. 9.21D+02) GOTO 3
       IF (T7 .LT. 1.0D+03) GOTO 5
       F=(3.381D+02-3.31D+00*(T7-1.0D+03)+9.83D-03*(T7-9.9999999D+02
     & )**1.96D+00-7.11D+00*DSIN(0.034*(T7-1.0D+03)))/(-4.187D+00)
       GOTO 4
1      F=1.38D+00*T7-1.499D+03
       GOTO 4
2      F=1.65786D+00*T7-1.581D+03
       GOTO 4
3      F=1.30089D+00*T7-1.331D+03
       GOTO 4
5      F=-1.0D+00*(1.20D+02-(T7-9.4D+02)*(0.73333333333D+00))
4      PEARLENERGY=(1.41D+02*T10 + F)*4.187D+00
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY (J/MOLE) IN AUSTENITE:
C INCLUDE T10,T20 PREVENT ERROR ON LINUX JUN/2003 BY T. OKUMURA
C CORRECT COPPER MOLE WEIGHT JUN/2003 BY T. OKUMURA
       SUBROUTINE PEARLOMEGA(W,T10,T20)
       COMMON/TRANS/C0,W0,K
       DOUBLE PRECISION C(8),W,P(8),B1,B2,Y(8),T10,T20,B3,XONE,
     & C0(8),W0(8),K(8)
       INTEGER I,U
       DO 1 I=1,8
        C(I)=W0(I)
1      CONTINUE
       B3=0.0D+00

C CORRECT C(5):Cr C(6):Mo
C       C(5)=C(5)/95.94D+00
C       C(6)=C(6)/52.0D+00

       C(8)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)
       C(8)=100.0D+00-C(8)
       C(8)=C(8)/55.84D+00
       C(1)=C(1)/12.0115D+00
       C(2)=C(2)/28.09D+00
       C(3)=C(3)/54.94D+00
       C(4)=C(4)/58.71D+00
       C(6)=C(6)/95.94D+00
       C(5)=C(5)/52.0D+00
       C(7)=C(7)/63.55D+00

       B1=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)
       DO 107 U=2,7
        Y(U)=C(U)/C(8)
107    CONTINUE
       DO 106 U=1,8
        C(U)=C(U)/B1
106    CONTINUE
       XONE=C(1)
       B2=0.0D+00
C CORRECT VALUES FOR COPPER
       T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*16

       T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)+4.5*Y(7)
C       T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*7
C       T20=-3*Y(2)-37.5*Y(3)-6*Y(4)-26*Y(5)-19*Y(6)-4.5*Y(7)

       P(2)=2013.0341+763.8167*C(2)+45802.87*C(2)**2-280061.63*C(2)**3
     & +3.864D+06*C(2)**4-2.4233D+07*C(2)**5+6.9547D+07*C(2)**6
       P(3)=2012.067-1764.095*C(3)+6287.52*C(3)**2-21647.96*C(3)**3-
     & 2.0119D+06*C(3)**4+3.1716D+07*C(3)**5-1.3885D+08*C(3)**6
       P(4)=2006.8017+2330.2424*C(4)-54915.32*C(4)**2+1.6216D+06*C(4)**3
     & -2.4968D+07*C(4)**4+1.8838D+08*C(4)**5-5.5531D+08*C(4)**6
       P(5)=2006.834-2997.314*C(5)-37906.61*C(5)**2+1.0328D+06*C(5)**3
     & -1.3306D+07*C(5)**4+8.411D+07*C(5)**5-2.0826D+08*C(5)**6
       P(6)=2012.367-9224.2655*C(6)+33657.8*C(6)**2-566827.83*C(6)**3
     & +8.5676D+06*C(6)**4-6.7482D+07*C(6)**5 +2.0837D+08*C(6)**6
       P(7)=2011.0D0

       DO 108 U=2,7
        B3=B3+P(U)*Y(U)
        B2=B2+Y(U)
108    CONTINUE
       IF ((B2 .LT.1D-40).OR.(B3.LT.1D-40)) GOTO 455
       W=(B3/B2)*4.187
       GOTO 456

455    W=8054.0

456    RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE TO CALCULATE COMPOSITION AT THE AUSTENITE / AUSTENITE+FERRITE INTERFACE:
       SUBROUTINE PEQGA0(T,X,Y,IM)
       IMPLICIT REAL*8 (A-H,K-Z)
       DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),
     & K(8),XXX(3),FUN1(3),G(8),G2(8),G3(8),G4(8),GM(8),
     & LFX(4,8),AC(8),E(8,8),EA(8,8),G0(8)
       COMMON/TRANS/C0,W0,K
       COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC,
     & GFE3C,G3,G4,WW,E,EA,G,G0
       R=8.314D+00
       X1=0.01D+00
       DX1=1.0D-06
200    XXX(1)=X1-DX1
       XXX(2)=X1+DX1
       XXX(3)=X1
       DO 220 I=1,3
        X1=XXX(I)
C USING DATA FROM KIRKALDY
C       A1=DEXP(G4(1)/R/T+E(1,1)*X1)/
C     &  (1.0D+00+EA(1,1)*X1*DEXP(G4(1)/R/T))
        A1=DEXP(G0(1)/R/T+E(1,1)*X1)/
     &  (1.0D+00+EA(1,1)*X1*DEXP(G0(1)/R/T))
        Y1=A1*X1
        X2=(1.0D+00-X1)*K(IM)/(1.0D+00+K(IM))
        Y2=(1.0D+00-Y1)*K(IM)/(1.0D+00+K(IM))
        X0=1.0D+00-X1-X2
        Y0=1.0D+00-Y1-Y2
        FFG=R*T*DLOG(X0)-R*T/2.0*E(1,1)*X1*X1
     &  -R*T/2.0*E(IM,IM)*X2*X2
     &  -R*T*E(1,IM)*X1*X2
        FFA=R*T*DLOG(Y0)-R*T/2.0*EA(1,1)*Y1*Y1
     &      -R*T/2.0*EA(IM,IM)*Y2*Y2
     &      -R*T*E(1,IM)*Y1*Y2
        F2G=R*T*DLOG(X2)+R*T*(E(1,IM)*X1+E(IM,IM)*X2)
        F2A=R*T*DLOG(Y2)+R*T*(E(1,IM)*Y1+EA(IM,IM)*Y2)
C USING DATA FROM KIRKALDY
C        FUN1(I)=X0*(G4(8)+FFG-FFA)+X2*(G4(IM)+F2G-F2A)
        FUN1(I)=X0*(G4(8)+FFG-FFA)+X2*(G0(IM)+F2G-F2A)
220    CONTINUE
       IF (FUN1(3) .LT. 1.0D-06) GOTO 300
       DUMMY1=X1-2.0*DX1*FUN1(3)/(FUN1(2)-FUN1(1))
       IF (DUMMY1 .LT. 0.0D+00) THEN
        DUMMY1=X1/2.0D+00
       ENDIF
       X1=DUMMY1
       GOTO 200
300    X(1)=X1
       X(IM)=X2
       Y(1)=Y1
       Y(IM)=Y2
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE TO CALCULATE THE PARAEQUILIBRIUM CHEMISTRY AT THE
C AUSTENITE / AUSTENITE+CEMENTITE INTERFACE:
       SUBROUTINE PEQGC0(T,X,Y,IM)
       IMPLICIT REAL*8 (A-H,K-Z)
       DOUBLE PRECISION X(8),Y(8),C0(8),W0(8),WW(8),
     & K(8),X1(3),FUN(3),G(8),G2(8),G3(8),G4(8),GM(8),
     & LFX(4,8),AC(8),E(8,8),EA(8,8),G0(8)
       COMMON/TRANS/C0,W0,K
       COMMON/HILL/G2,LFX,LCV,GM,AC,DDG,GFEC,
     & GFE3C,G3,G4,WW,E,EA,G,G0
       R=8.314D+00
       XGA=0.005D+00
       DUMMY=1.0D-08
100    X1(1)=XGA-DUMMY
       X1(2)=XGA+DUMMY
       X1(3)=XGA
       DO 10 I=1,3
        XX=0.0D+00
        FUN(I)=0.0D+00
        KK=K(IM)
        Y(IM)=KK/(1+KK)
        X(IM)=(1-X1(I))*Y(IM)
        X(1)=X1(I)
        X(8)=1.0D+00-X(1)-X(IM)
        Y(8)=1.0D+00-Y(IM)
        DENO=1.0D+00
        MFE3C=R*T*DLOG(Y(8))
        MFE=R*T*DLOG(X(8)/DENO)
     &  -R*T/2.0*E(1,1)*X(1)*X(1)/DENO/DENO
        MC=R*T/3.0*DLOG(X(1)/DENO)
     &  +R*T/3.0*E(1,1)*X(1)/DENO
        MFE3C=MFE3C+(1-Y(8))*WW(IM)*Y(IM)
        MFE=MFE-R*T/2.0*E(IM,IM)*X(IM)*X(IM)/DENO/DENO
        MFE=MFE-R*T*X(1)*E(1,IM)*X(IM)/DENO/DENO
        MC=MC+R*T/3.0*E(1,IM)*X(IM)/DENO
        MX= R*T*DLOG(X(IM)/DENO)
     &  +R*T*(E(1,IM)*X(1)+E(IM,IM)*X(IM))/DENO
        MM3C= R*T*DLOG(Y(IM))
     &  +Y(8)*WW(IM)-Y(8)*WW(IM)*Y(IM)
        FUN(I)=X(8)*(GFE3C+MFE3C-MFE-MC)
        FUN(I)=FUN(I)+X(IM)*(G(IM)+MM3C-MX-MC)
10     CONTINUE
       IF(ABS(FUN(3)) .LT. 1.0D-6) GOTO 20
       XGA=X1(3)-2.0*DUMMY*FUN(3)/(FUN(2)-FUN(1))
       GOTO 100
20     XGA=X1(3)
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE USED TO FIND THE MAXIMUM POSSIBLE GROWTH RATE OF WIDMANSTATTEN FERRITE:
       SUBROUTINE PROGC(VMAX,TEMPC,XMAX,WDIFF,W,XBAR)
       IMPLICIT REAL*8 (A-H,K-Y), INTEGER(I,J,Z)
       DOUBLE PRECISION DIFF(1000),CARB(1000)
C CAPCON=CAPILLARITY CONSTANT.
C XMAXR=EQULIBRIUM CONCENTRATION OF CARBON IN AUSTENITE AT PLATE TIP OF FINITE RADIUS.
       IDIFFFLAG=0
       R=8.31432D+00
       RADIUS=0.0
       VMAX=0.0
       M1=0.00
       T=TEMPC
       CTEMP=T
       T=T+273.00D+00
       II2=0
       XALPHA=XALPH(T)
       CALL RRAD(RADIUS,XMAX,XALPHA,XBAR,T,R,XMAXR,W)
       DO 999 II=1,1000
        IF (II.EQ.735) THEN
         AA=AA
         BB=BB
        ENDIF
        CARB(1)=XBAR
        IF (II .GT. 1) GOTO 1000
        GOTO 1001
1000    CARB(II)=CARB(II-1)+0.0001D+00
        IF (CARB(II) .GT. XMAXR) GOTO 1002
1001    X=CARB(II)
        II2=II2+1
        CALL DIFFUS(DIFF(II),X,T,WDIFF)
C IF DIFFUSION COEFFICIENT FOUND TO BE NEGATIVE AT HIGH CARBON CONCENTRATIONS AND VERY LOW
C TEMPERATURES: 
        IF ((DIFF(II).LT.0.0).AND.(IDIFFFLAG.EQ.0)) THEN
         DIFFMAX=DIFF(II-5)
         DIFF(II-4)=DIFFMAX
         DIFF(II-3)=DIFFMAX
         DIFF(II-2)=DIFFMAX
         DIFF(II-1)=DIFFMAX
         DIFF(II)=DIFFMAX
         IDIFFFLAG=1
        ELSEIF (IDIFFFLAG.EQ.1) THEN
         DIFF(II)=DIFFMAX
        ENDIF
        IF (DIFF(II).LT.0.0) THEN
         WRITE(*,*) "DIFFUS ERROR !",CTEMP,X,DIFF(II)
         PAUSE
        ENDIF
999    CONTINUE
1002   II3=0
       CALL TRAPE(CARB,DIFF,ANS,II2)
       ANS=ANS/(XMAXR-XBAR)
       CALL VEL4(VMAX,ANS,RADIUS,XMAX,XBAR,XALPHA)
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE TO READ ALLOY COMPOSITION:
C INCLUDING COPPER (JUN/2003)
       SUBROUTINE RCOMP(C,J6,JYES)
       DOUBLE PRECISION C(9)
       INTEGER J6,JYES
       IF(JYES .EQ. 1)GOTO 1
       IF(JYES .EQ. 2)GOTO 2
       IF(JYES .EQ. 3)GOTO 3
       IF(JYES .EQ. 4)GOTO 4
       IF(JYES .EQ. 5)GOTO 5
       IF(JYES .EQ. 6)GOTO 6
       IF(JYES .EQ. 7)GOTO 7
       IF(JYES .EQ. 8)GOTO 9
1      WRITE(*,707)
707    FORMAT(' CARBON WT % ?')
       CALL REED(C(1))
       IF(C(1) .LT. 0.1D-03 .OR. C(1) .GT. 2.0D+00)
     & CALL BOUND(C(1),0.1D-03,2.0D+00)
       IF(JYES .NE. 0) GOTO 8
2      WRITE(*,708)
       CALL REED(C(2))
       IF(C(2) .LT. 0.0D+00 .OR. C(2) .GT. 2.5D+00)
     & CALL BOUND(C(2),0.0D+00,2.5D+00)
708    FORMAT(' SILICON WT % ?')
       IF(JYES .NE. 0) GOTO 8
3      WRITE(*,709)
       CALL REED(C(3))
       IF(C(3) .LT. 0.0D+00 .OR. C(3) .GT. 3.5D+00)
     & CALL BOUND(C(3),0.0D+00,3.5D+00)
709    FORMAT(' MANGANESE WT % ?')
       IF(JYES .NE. 0) GOTO 8
4      WRITE(*,710)
       CALL REED(C(4))
       IF(C(4) .LT. 0.0D+00 .OR. C(4) .GT. 4.5D+00)
     & CALL BOUND(C(4),0.0D+00,4.5D+00)
710    FORMAT(' NICKEL WT % ?')
711    FORMAT(' MOLYBDENUM WT % ?')
       IF(JYES .NE. 0) GOTO 8
5      WRITE(*,711)
       CALL REED(C(5))
       IF(C(5) .LT. 0.0D+00 .OR. C(5) .GT. 1.5D+00)
     & CALL BOUND(C(5),0.0D+00,1.5D+00)
712    FORMAT(' CHROMIUM WT % ?')
       IF(JYES .NE. 0) GOTO 8
6      WRITE(*,712)
       CALL REED(C(6))
       IF(C(6) .LT. 0.0D+00 .OR. C(6) .GT. 3.5D+00)
     & CALL BOUND(C(6),0.0D+00,3.5D+00)
       IF(JYES .NE. 0) GOTO 8
7      WRITE(*,713)
713    FORMAT(' VANADIUM WT % ?')
       CALL REED(C(7))
       IF(C(7) .LT. 0.0D+00 .OR. C(7) .GT. 1.5D+00)
     & CALL BOUND(C(7),0.0D+00,1.5D+00)
9      WRITE(*,714)
 714   FORMAT(' COPPER WT % ?')
       CALL REED(C(8))
       IF(C(8) .LT. 0.0D+00 .OR. C(8) .GT. 4.0D+00)
     & CALL BOUND(C(8),0.0D+00,4.0D+00)
8      RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE TO ASCERTAIN THAT INPUT IS A REAL NUMBER:
       SUBROUTINE REED(A)
       DOUBLE PRECISION A
996    READ(*,*,ERR=999)A
       GOTO 998
999    WRITE(*,997)
997    FORMAT(' INCORRECT INPUT. TRY AGAIN'/)
       GOTO 996
998    RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE TO ASCERTAIN THAT INPUT IS AN INTEGER:
       SUBROUTINE REEDI(I)
       INTEGER I
996    READ(*,*,ERR=999)I
       GOTO 998
999    WRITE(*,997)
997    FORMAT(' INCORRECT INPUT. TRY AGAIN'/)
       GOTO 996
998    RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE USED TO FIND THE SOLUTION TO TRIVEDI'S EQUATION (SJJ JONES). ROOT FINDING
C ALGORITHM DEVELOPED BY VAN WIJNGAARDEN ET AL. AS GIVEN IN NUMERICAL RECIPES - THE ART
C OF SCIENTIFIC COMPUTING, CAMBRIDGE UNIVERSITY PRESS, 1986:
       SUBROUTINE ROOTFINDVEL(RATIO,OMEGA,PECLET)
       IMPLICIT DOUBLE PRECISION (A-H,K-Z), INTEGER (I-J)
       DOUBLE PRECISION ALLPECLET(2)
       STARTPEC=1.0D-20
C BRACKET THE TWO POSSIBLE ROOTS:
       FACTOR=1.0
       ADJUST=1.0D-5
       IEND=100
22     IFLAG=0
       DO 23 I23=1,IEND,1
        BRKPEC=STARTPEC+(ADJUST*FACTOR*I23)
        TEST=FUNCTRIV(BRKPEC,OMEGA,RATIO)
        IF ((TEST.GT.0.0).AND.(IFLAG.EQ.0)) THEN
         PECPOSIT=BRKPEC
         IFLAG=1
        ENDIF
        IF ((IFLAG.EQ.1).AND.(TEST.LT.0.0)) THEN
         PECNEGAGAIN=BRKPEC
         INHUNTS=2
         GOTO 24
        ENDIF
        IF (BRKPEC.GT.600.0) THEN
         IF (IFLAG.EQ.0) THEN
          WRITE(*,*) 'ERROR - TRIVEDI ROOT !'
          PAUSE
         ELSEIF (IFLAG.EQ.1) THEN
          INHUNTS=1
          WRITE(*,*) 'ERROR - BRACKETING TRIVEDI ROOTS !'
          PAUSE
         ENDIF
        ENDIF
        IF (I23.EQ.IEND) THEN
         FACTOR=FACTOR*2.0
         GOTO 22
        ENDIF
23     CONTINUE
C FIND ROOTS ROOTS:
24     DO 30 I30=1,INHUNTS,1
        IF (I30.EQ.1) THEN
         ZX1=STARTPEC
         ZX2=PECPOSIT
        ELSEIF (I30.EQ.2) THEN
         ZX1=PECPOSIT
         ZX2=PECNEGAGAIN
        ENDIF
        ZTOL=1.0D-20
        ITMAX=1000
        ZEPS=3.0D-9
        ZA=ZX1
        ZB=ZX2
        ZFA=FUNCTRIV(ZA,OMEGA,RATIO)
        ZFB=FUNCTRIV(ZB,OMEGA,RATIO)
        IF(ZFB*ZFA.GT.0.0) THEN
         PAUSE 'ROOT MUST BE BRACKETED FOR ZBRENT'
        ENDIF
        ZFC=ZFB
        DO 911 ITER=1,ITMAX
         IF(ZFB*ZFC.GT.0.0) THEN
          ZC=ZA
          ZFC=ZFA
          ZD=ZB-ZA
          ZE=ZD
         ENDIF
         IF(ABS(ZFC).LT.ABS(ZFB)) THEN
          ZA=ZB
          ZB=ZC
          ZC=ZA
          ZFA=ZFB
          ZFB=ZFC
          ZFC=ZFA
         ENDIF
         ZTOL1=2.0*ZEPS*ABS(ZB)+0.5*ZTOL
         ZXM=0.5*(ZC-ZB)
         IF(ABS(ZXM).LE.ZTOL1.OR.ZFB.EQ.0.0) THEN
          ZBRENT=ZB
          GOTO 978
         ENDIF
         IF (ABS(ZE).GE.ZTOL1.AND.ABS(ZFA).GT.ABS(ZFB)) THEN
          ZS=ZFB/ZFA
          IF(ZA.EQ.ZC) THEN
           ZP=2.0*ZXM*S
           ZQ=1.0-ZS
          ELSE
           ZQ=ZFA/ZFC
           ZR=ZFB/ZFC
           ZP=ZS*(2.0*ZXM*ZQ*(ZQ-ZR)-(ZB-ZA)*(ZR-1.0))
           ZQ=(ZQ-1.0)*(ZR-1.0)*(ZS-1.0)
          ENDIF
          IF (ZP.GT.0) ZQ=-ZQ
          ZP=ABS(ZP)
          IF(2.0*ZP.LT.MIN(3.0*ZXM*ZQ-ABS(ZTOL1*ZQ),ABS(ZE*ZQ)))THEN
           ZE=ZD
           ZD=ZP/ZQ
          ELSE
           ZD=ZXM
           ZE=ZD
          ENDIF
         ELSE
          ZD=ZXM
          ZE=ZD
         ENDIF
         ZA=ZB
         ZFA=ZFB
         IF (ABS(ZD).GT.ZTOL1) THEN
          ZB=ZB+ZD
         ELSE
          ZB=ZB+SIGN(ZTOL1,ZXM)
         ENDIF
         ZFB=FUNCTRIV(ZB,OMEGA,RATIO)
911     CONTINUE
        PAUSE 'ERROR - EXCEEDING MAXIMUM ITERATIONS !'
        ZBRENT=ZB
978     ALLPECLET(I30)=ZBRENT
30     CONTINUE
       IF (ALLPECLET(1).GE.ALLPECLET(2)) THEN
        PECLET=ALLPECLET(1)
       ELSE
        PECLET=ALLPECLET(2)
       ENDIF
345    RETURN
       END
C
C-------------------------------------------------------------------------------
C

       SUBROUTINE RRAD(RADIUS,XMAX,XALPHA,XBAR,T,R,XMAXR,W)
       DOUBLE PRECISION RADIUS,XMAX,XBAR,T,R,SIG,MOLVOL,XMAXR,
     & XALPHA,RAD,OMEG,CAPCON,EPSI,W,DCG
C SIG=INTERFACIAL ENERGY (J/M^2).
C MOLVOL=MOLAR VOLUME OF FERRITE.
C RADIUS IS THE CRITICAL RADIUS FOR ZERO GROWTH.
C RAD IS THE RATIO OF THE ACTUAL RADIUS TO THE CRITICAL RADIUS.
       SIG=0.2
       MOLVOL=7.0894317D-06*(1.0D+00+3.549D-05*(T-298.0D+00))
       EPSI=XMAX*DCG(XMAX,T,W,R)
       CAPCON=(SIG*MOLVOL/(R*T))*((1.0D+00-XMAX)/(XALPHA-XMAX))
     & *EPSI
       RADIUS=CAPCON*XMAX/(XBAR-XMAX)
       OMEG=(XMAX-XBAR)/(XMAX-XALPHA)
       RAD=0.2026D+01-0.1917D+01*OMEG-0.8953D+00*OMEG*OMEG
     & +0.3670D+01*OMEG*OMEG*OMEG
     & -0.2519D+01*OMEG*OMEG*OMEG*OMEG
       RAD=10.00D+00**RAD
       RAD=RADIUS*RAD
       XMAXR=XMAX*(1.0D+00-(CAPCON/RAD))
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE WHICH COMPUTES THE VECTOR OF INTEGRAL VALUES FOR A GIVEN GENERAL TABLE OF
C ARGUMENT AND FUNCTION VALUES:
       SUBROUTINE TRAPE(X,Y,ANS,NDIM)
       DOUBLE PRECISION X(1000),Y(1000),AZ(1000)
       DOUBLE PRECISION SUM1,SUM2,ANS
C BASED ON WORK BY F.B.HILDEBRAND, INTRODUCTION TO NUMERICAL ANALYSIS, MCGRAW-HILL,
C NEW YORK, 1956, P.75. FROM SSP (IBM), CF. PLUS/1000 X019 HO <881109.1345>
       SUM2=0.D+00
       IF(NDIM-1)4,3,1
1      DO 2 I=2,NDIM
        SUM1=SUM2
        SUM2=SUM2+.5D+00*(X(I)-X(I-1))*(Y(I)+Y(I-1))
2      AZ(I-1)=SUM1
3      AZ(NDIM)=SUM2
       ANS=SUM2
4      RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE TO ESTIMATE THE TTT CURVES FOR RECONSTRUCTIVE AND DISPLACIVE TRANSFORMATIONS:
       SUBROUTINE TTTT(T,GMAX,SHEART,DIFFT)
       IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
       SHEART=DEXP((0.2432D+06/(8.31432*T))-0.135D+03+20.0*DLOG(T)
     & -5*DLOG(DABS(GMAX)))
       DIFFT=DEXP((0.6031D+06/(8.31432*T))-0.1905D+03+20.0*DLOG(T)
     & -4*DLOG(DABS(GMAX)))
       RETURN
       END
C
C-------------------------------------------------------------------------------
C
C FUNCTION TO CALCULATE THE BOUNDARY DIFFUSION CONTROLLED GROWTH RATE OF PEARLITE:
       DOUBLE PRECISION FUNCTION VBD(T,XGA,XGC,SC,XBAR,S,IM)
       IMPLICIT REAL*8 (A-H,K-Z)
       DOUBLE PRECISION CK(8),Q(8)
       CK(3)=1.126D-08
       CK(5)=2.455D-09
       Q(3)=141178.0D+00
       Q(5)=138516.0D+00
       R=8.314D+00
       SA=S*7.0D+00/8.0D+00
       SCEM=S/8.0D+00
       KDD=CK(IM)*DEXP(-Q(IM)/R/T)
       VBD=12.0D+00*KDD/SA/SCEM*(XGA-XGC)/XBAR*(1.0D+00-SC/S)
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C FUNCTION TO CALCULATE THE PARAEQUILIBRIUM CARBON DIFFUSION CONTROLLED GROWTH RATE OF
C PEARLITE:
       DOUBLE PRECISION FUNCTION VD(XGA,XGC,XAG,SC,D,S)
       IMPLICIT REAL*8 (A-H,K-Z)
       A=0.72D+00
       SA=S*7.0D+00/8.0D+00
       SCEM=S/8.0D+00
       CCEM=0.25D+00
       VD=D/A*S**2/SA/SCEM*(XGA-XGC)/(CCEM-XAG)/S*(1.0D+00-SC/S)
       RETURN
       END
C
C-------------------------------------------------------------------------------
C
C SUBROUTINE USED TO FIND THE MAXIMUM POSSIBLE GROWTH RATE OF WIDMANSTATTEN FERRITE:
       SUBROUTINE VEL4(VMAX,ANS,RADIUS,XMAX,XBAR,XALPHA)
       IMPLICIT REAL*8(A-H,K-Z), INTEGER(I,J)
C MODELS THE DIFFUSION-CONTROLLED GROWTH OF PARABOLIC CYLINDERS (TRIVEDI ANALYSIS):
       IF(XBAR .LE. XALPHA) THEN
        OMEG=0.99D+00
       ELSE
        OMEG=(XMAX-XBAR)/(XMAX-XALPHA)
       ENDIF
       ANS=ANS*1.0D-04
C THE FOLLOWING POLYNOMIAL HAS BEEN DERIVED FOR THE OMEG RANGE 0.4 TO 1:
       LOG10RATIO=0.2026D+01-0.1917D+01*OMEG-0.8953D+00*OMEG*OMEG
     & +0.3670D+01*OMEG*OMEG*OMEG
     & -0.2519D+01*OMEG*OMEG*OMEG*OMEG
       RATIO=10.0**LOG10RATIO
C CALL NEW SUBROUTINE TO FIND MAXIMUM PECLET NUMER FOR THIS SYSTEM (SJ JONES):
       CALL ROOTFINDVEL(RATIO,OMEG,PECLET)
       RAD=RATIO*RADIUS
       VMAX=PECLET*(2.0D+00*ANS)/RAD
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE USED TO WRITE ERROR MESSAGES FOR SL-MATH SUBROUTINES (5736-XM7 COPYRIGHT IBM
C CORPORATION 1971):
       SUBROUTINE WIER(IER,NO)
       INTEGER IER,NO
       WRITE (3,1) NO,IER
1      FORMAT (//' ***** DKO',I5,' RAISED ERROR INDICATOR TO ',I4,
     &     ' *****'///)
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C FUNCTION WHICH DETERMINES THE UNIVERSAL NUCLEATION FUNCTION FOR DISPLACIVE
C TRANSFORMATION:
       DOUBLE PRECISION FUNCTION WSFUN(KTEMP)
       DOUBLE PRECISION KTEMP
C BHADESHIA, ACTA METALL. VOL. 29, 1981, P.1117-1130.
       WSFUN=3.251745029D+00*(KTEMP-273.0D+00)-2183.014D+0
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C SUBROUTINE WHICH CALCULATES THE WIDMANSTATTEN FERRITE NUCLEATION RATE:
C
      SUBROUTINE WSNUCRATE(WSI1AREA,WSI1VOL,SV,T,GMAX)
      IMPLICIT REAL*8 (A-H,K-Z), INTEGER (I,J)   
C  
C BASED ON BHADESHIA AND REES (MAT. SCI. AND TECH. 1992, VOL.8, P985-993):  
C
C U IS THE VOLUME OF A TYPICAL BAINITIC SUBUNIT
           U=(10.0D-06)*(10.0D-06)*(0.2D-06)
           CONSTR=2540.0
           R=8.31432
           K2=2.065D+04
           K1DASHU=33.90D+06
           K1DASH =U*K1DASHU
           LBAR=2.0/SV
           K1=1.0/(LBAR*K1DASH)
           WSI1VOL=K1*
     &     DEXP(
     &     ( ((-1.0*K2)/R/T)-((K2*GMAX)/CONSTR/R/T) )
     &     )
C CONVERT UNITS TO PER UNIT AREA OF BOUNDARY:
           WSI1AREA=WSI1VOL/SV 
C
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C Estimation of the Widmanstatten ferrite start temperature
C
       SUBROUTINE WSTINE(DFPRO,WS,WS1,J8)
         IMPLICIT REAL*8(A-H,K-Z),INTEGER(I,J)
         DOUBLE PRECISION DFPRO(40)
C
       J=0
       DO 1 I=1,J8
            KTEMP=473.00D+00+(I-1)*20.0D+00
            V14=WSFUN(KTEMP)
C
C Stored energy for the growth of Widmanstatten ferrite, 50 J/mol
C
            IF(DFPRO(I) .GT. -50.0D+00) GOTO 1
C
C Ws1 is the lower intersection of the GN and FPRO curves
C
            IF(DFPRO(I) .LE. V14 .AND. J .EQ. 0) THEN
                 WS1=KTEMP-273.0D+00
            ENDIF
            IF(DFPRO(I) .GE. V14)J=1
C
C Ws is the upper intersection of the GN and FPRO curves
C
            IF(DFPRO(I) .LE. V14) WS=KTEMP-273.00D+00
1      CONTINUE
C
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C Function used to determine the interface chemistry
C
       DOUBLE PRECISION FUNCTION X2AC0(T,X1,A,E11,E12)
       IMPLICIT REAL*8 (A-H,K-Z)
       X2AC0=(DLOG(A)-DLOG(X1)-E11*X1)/E12
       RETURN
       END
C
C-------------------------------------------------------------------------------
C

C FUNCTION GIVING THE EQUILIBRIUM MOLE FRACTION OF CARBON IN FERRITE:
       DOUBLE PRECISION FUNCTION XALPH(T)
       DOUBLE PRECISION T,CTEMP
       CTEMP=(T-273.0D+00)/900.0D+00
       XALPH=0.1528D-02-0.8816D-02*CTEMP+0.2450D-01*CTEMP*CTEMP
     & -0.2417D-01*CTEMP*CTEMP*CTEMP+
     & 0.6966D-02*CTEMP*CTEMP*CTEMP*CTEMP
       RETURN
       END
C
C-------------------------------------------------------------------------------

